#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2021 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Luis Cañas-Díaz <lcanas@bitergia.com>
#     Alvaro del Castillo <acs@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#

import json
import logging
import re

from grimoire_elk.elk import get_ocean_backend
from grimoire_elk.utils import get_connector_from_name, get_elastic
from grimoire_elk.enriched.utils import grimoire_con

logger = logging.getLogger(__name__)


class Task():
    """ Basic class shared by all tasks """

    NO_BACKEND_FIELDS = ['enriched_index', 'raw_index', 'es_collection_url',
                         'collect', 'pair-programming', 'fetch-archive',
                         'studies', 'node_regex', 'anonymize']
    PARAMS_WITH_SPACES = ['blacklist-jobs']

    def __init__(self, config, sortinghat_client=None):
        self.backend_section = None
        self.config = config
        self.conf = config.get_conf()
        self.client = sortinghat_client

        sortinghat = self.conf.get('sortinghat', None)
        self.db_sh = sortinghat['database'] if sortinghat else None
        self.db_user = sortinghat['user'] if sortinghat else None
        self.db_password = sortinghat['password'] if sortinghat else None
        self.db_host = sortinghat['host'] if sortinghat else '127.0.0.1'
        self.db_path = sortinghat.get('path', None) if sortinghat else None
        self.db_port = sortinghat.get('port', None) if sortinghat else None
        self.db_ssl = sortinghat.get('ssl', False) if sortinghat else False
        self.db_verify_ssl = sortinghat.get('verify_ssl', True) if sortinghat else True
        self.db_tenant = sortinghat.get('tenant', True) if sortinghat else None
        self.db_unaffiliate_group = sortinghat['unaffiliated_group'] if sortinghat else None

        self.grimoire_con = grimoire_con(conn_retries=12)  # 30m retry

    @staticmethod
    def anonymize_url(url):
        anonymized = re.sub('://.*@', '://', url)

        return anonymized

    @staticmethod
    def load_aliases_from_json(aliases_json):
        with open(aliases_json, 'r') as f:
            try:
                aliases = json.load(f)
            except Exception as ex:
                logger.error(ex)
                raise

        return aliases

    def is_backend_task(self):
        """
        Returns True if the Task is executed per backend.
        i.e. SortingHat unify is not executed per backend.
        """
        return True

    def execute(self):
        """ Execute the Task """
        logger.debug("A bored task. It does nothing!")

    @classmethod
    def get_backend(self, backend_section):
        # To support the same data source with different configs
        # remo:activitites is like remo but with an extra param
        # category = activity
        backend = backend_section.split(":")[0]
        return backend

    def set_backend_section(self, backend_section):
        self.backend_section = backend_section

    def _extract_repo_tags(self, backend_section, repo, tag_type="labels"):
        """Extract the tags declared in the repositories within the projects.json, and remove them to
        avoid breaking already existing functionalities.

        :param backend_section: name of the backend section
        :param repo: repo url in projects.json
        """
        backend = self.get_backend(backend_section)
        connector = get_connector_from_name(backend)
        ocean = connector[1]

        processed_repo, tags_lst = ocean.extract_repo_tags(repo, tag_type)
        return processed_repo, tags_lst

    def _compose_p2o_params(self, backend_section, repo):
        # get p2o params included in the projects list
        params = {}

        backend = self.get_backend(backend_section)
        connector = get_connector_from_name(backend)
        ocean = connector[1]

        # First add the params from the URL, which is backend specific
        params = ocean.get_p2o_params_from_url(repo)

        return params

    def _compose_perceval_params(self, backend_section, repo):
        backend = self.get_backend(backend_section)
        connector = get_connector_from_name(backend)
        ocean = connector[1]

        # First add the params from the URL, which is backend specific
        params = ocean.get_perceval_params_from_url(repo)

        # Now add the backend params included in the config file
        for p in self.conf[backend_section]:
            if p in self.NO_BACKEND_FIELDS:
                # These params are not for the perceval backend
                continue

            section_param = self.conf[backend_section][p]
            if not section_param:
                logger.warning("Empty section %s", p)
                continue

            # If param is boolean, no values must be added
            if isinstance(section_param, bool):
                params.append("--" + p) if section_param else None
            elif isinstance(section_param, list):
                # '--blacklist-jobs', 'a', 'b', 'c'
                # 'a', 'b', 'c' must be added as items in the list
                params.append("--" + p)
                list_params = section_param
                params += list_params
            else:
                params.append("--" + p)
                params.append(str(section_param))

        return params

    def _get_collection_url(self):
        es_col_url = self.conf['es_collection']['url']
        if self.backend_section and self.backend_section in self.conf:
            if 'es_collection_url' in self.conf[self.backend_section]:
                es_col_url = self.conf[self.backend_section]['es_collection_url']
        else:
            logger.warning("No config for the backend %s", self.backend_section)
        return es_col_url

    def _get_enrich_backend(self):
        json_projects_map = None
        clean = False
        connector = get_connector_from_name(self.get_backend(self.backend_section))

        if 'projects_file' in self.conf['projects']:
            json_projects_map = self.conf['projects']['projects_file']

        enrich_backend = connector[2](db_sortinghat=self.db_sh, json_projects_map=json_projects_map,
                                      db_user=self.db_user, db_password=self.db_password, db_host=self.db_host,
                                      db_port=self.db_port, db_path=self.db_path, db_ssl=self.db_ssl,
                                      db_verify_ssl=self.db_verify_ssl, db_tenant=self.db_tenant)
        elastic_enrich = get_elastic(self.conf['es_enrichment']['url'],
                                     self.conf[self.backend_section]['enriched_index'],
                                     clean, enrich_backend)
        enrich_backend.set_elastic(elastic_enrich)
        if 'pair-programming' in self.conf[self.backend_section]:
            enrich_backend.pair_programming = self.conf[self.backend_section]['pair-programming']

        if self.db_unaffiliate_group:
            enrich_backend.unaffiliated_group = self.db_unaffiliate_group

        return enrich_backend

    def _get_ocean_backend(self, enrich_backend):
        backend_cmd = None

        no_incremental = False
        clean = False

        from .task_projects import TaskProjects
        repos = TaskProjects.get_repos_by_backend_section(self.backend_section)
        if len(repos) == 1:
            # Support for filter raw when we have one repo
            repo = repos[0]
            repo, repo_labels = self._extract_repo_tags(self.backend_section, repo)
            p2o_args = self._compose_p2o_params(self.backend_section, repo)
            filter_raw = p2o_args['filter-raw'] if 'filter-raw' in p2o_args else None

            ocean_backend = get_ocean_backend(backend_cmd, enrich_backend, no_incremental, filter_raw)
        else:
            ocean_backend = get_ocean_backend(backend_cmd, enrich_backend, no_incremental)

        elastic_ocean = get_elastic(self._get_collection_url(),
                                    self.conf[self.backend_section]['raw_index'],
                                    clean, ocean_backend)
        ocean_backend.set_elastic(elastic_ocean)

        return ocean_backend

    def es_version(self, url):
        """Get Elasticsearch version.

        Get the version of Elasticsearch. This is useful because
        Elasticsearch and Kibiter are paired (same major version for 5, 6).

        :param url: Elasticseearch url hosting Kibiter indices
        :returns:   major version, as string
        """

        try:
            res = self.grimoire_con.get(url)
            res.raise_for_status()
            major = res.json()['version']['number'].split(".")[0]
        except Exception:
            logger.error("Error retrieving Elasticsearch version: " + url)
            raise
        return major

    @staticmethod
    def retain_data(retention_time, es_url, index):
        elastic = get_elastic(es_url, index)
        elastic.delete_items(retention_time)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Luis Cañas-Díaz <lcanas@bitergia.com>
#


import argparse
import json
import re
import sys

from datetime import datetime
from file_read_backwards import FileReadBackwards
from sirmordred.config import Config

HEALTHCHECK_CACHEFILE = '/tmp/.mordred_healthcheck'
HEALTHCHECK_DATEFORMAT = '%Y-%m-%d %H:%M:%S,%f'
HEALTHCHECK_DESCRIPTION = "Healthcheck for SirMordred"
HEALTHCHECK_EPILOG = "Software metrics for your peace of mind"
HEALTHCHECK_LOGREGEXP = '[2][0-9]{3}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}.*'


def main():
    logs_dir, match_string = parse_args()
    time_b = datetime.now()
    healthy, time_a = read_cache_file()
    error_found = False

    if healthy and (time_a is not None):
        # We discard to search if the cache file is not created, this way the container
        #  won't have to worry about old logs
        file_path = logs_dir + '/all.log'
        error_found = match_error_string(file_path, time_a, time_b, match_string)
        healthy = not error_found

    write_cache_file(healthy, time_b)

    if not healthy:
        sys.exit(1)


def parse_args():
    """ Parses arguments and returns a tuple with the path of the logs dir and the string to be found """

    parser = argparse.ArgumentParser(
        description=HEALTHCHECK_DESCRIPTION,
        epilog=HEALTHCHECK_EPILOG
    )

    parser.add_argument('-c', '--config', help='Configuration file',
                        type=str, default=['setup.cfg'],
                        dest='config_file')
    parser.add_argument('-s', '--string', help='String to match errors in log file',
                        type=str,
                        dest='match_string')

    args = parser.parse_args()

    config = Config(args.config_file)
    config_dict = config.get_conf()

    return config_dict['general']['logs_dir'], args.match_string


def read_cache_file():
    """Reads the content of the cache file with the data of the last execution.

    :returns: tuple with boolean and a datetime if file is found. True, None if any exception is raised
    """
    default_output = True, None
    try:
        with open(HEALTHCHECK_CACHEFILE, 'r') as f:
            cache = json.load(f)
            return cache['healthy'], datetime.strptime(cache['time'], HEALTHCHECK_DATEFORMAT)

    except FileNotFoundError:
        return default_output
    except json.decoder.JSONDecodeError:
        return default_output
    except KeyError:
        return default_output


def match_error_string(file_path, time_a, time_b, match_string):
    """Searches a string in a log file for all the lines where date is between time_a and time_b

    :param logs_dir: path of the directory where log file is located
    :param time_a: time which defines the starting point of the time frame
    :param time_b: time which defines the ending point of the time frame
    :param match_string: string to be search in any log lines
    :returns: If the string is found it returns True. False is returned otherwise
    """
    with FileReadBackwards(file_path, encoding="utf-8") as frb:
        for line in frb:
            match = re.match(HEALTHCHECK_LOGREGEXP, line)
            if not match:
                continue

            columns = line.split(' ')
            str_date = columns[0] + ' ' + columns[1]
            dt_line = datetime.strptime(str_date, HEALTHCHECK_DATEFORMAT)
            if dt_line < time_a or dt_line > time_b:
                break

            if line.find(match_string) > 0:
                return True

    return False


def write_cache_file(is_healthy, time):
    """Stores a JSON with a boolean and a datetime timestamp in the cache file

    :param is_healthy: boolean
    :param time: datetime
    """

    cache_content = {}
    cache_content['time'] = time.strftime(HEALTHCHECK_DATEFORMAT)
    cache_content['healthy'] = is_healthy

    with open(HEALTHCHECK_CACHEFILE, 'w+') as f:
        f.write(json.dumps(cache_content))


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2020 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Valerio Cosentino <valcos@bitergia.com>
#

import argparse
import os
import sys

import panels
from sirmordred.config import Config
from sirmordred.task_panels import TaskPanels, TaskPanelsMenu


def get_params_parser():
    """Parse command line arguments"""

    parser = argparse.ArgumentParser(add_help=False)

    parser.add_argument('-g', '--debug', dest='debug',
                        action='store_true',
                        help=argparse.SUPPRESS)

    parser.add_argument("--cfg", dest='cfg_path',
                        help="Configuration file path")
    parser.add_argument("--dashboards", action='store_true', dest='dashboards',
                        help="Upload dashboards")
    parser.add_argument("--menu", action='store_true', dest='menu',
                        help="Upload menu")

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)

    return parser


def get_params():
    """Get params to upload dashboards and menu"""

    parser = get_params_parser()
    args = parser.parse_args()

    return args


def get_sigils_path():
    sigils_path = panels.__file__.replace('panels/__init__.py', '')
    return sigils_path


def read_file(filename, mode='r'):
    with open(os.path.join(os.path.dirname(os.path.abspath(__file__)), filename), mode) as f:
        content = f.read()
    return content


def main():
    """This script allows to upload the dashboards in use in the setup.cfg and the top menu. It
    relies on the TaskPanels and TaskPanelsMenu classes.

    Examples:
        Upload dashboards and menu: panels_config --cfg ./setup.cfg --dashboards --menu
        Upload only the dashboards: panels_config --cfg ./setup.cfg --dashboards
        Upload only the top menu:   panels_config --cfg ./setup.cfg --menu
    """
    args = get_params()
    config = Config(args.cfg_path)

    upload_dashboards = args.dashboards
    upload_menu = args.menu

    if upload_dashboards:
        task = TaskPanels(config)
        task.execute()

    if upload_menu:
        task = TaskPanelsMenu(config)
        task.execute()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#   Valerio Cosentino <valcos@bitergia.com>
#

import argparse
import logging
import os

import colorlog
import sys

from sirmordred.config import Config
from sirmordred.task_collection import TaskRawDataCollection
from sirmordred.task_identities import TaskIdentitiesMerge
from sirmordred.task_enrich import TaskEnrich
from sirmordred.task_panels import TaskPanels, TaskPanelsMenu
from sirmordred.task_projects import TaskProjects
from sortinghat.cli.client import SortingHatClient

COLOR_LOG_FORMAT_SUFFIX = "\033[1m %(log_color)s "
LOG_COLORS = {'DEBUG': 'white', 'INFO': 'cyan', 'WARNING': 'yellow', 'ERROR': 'red', 'CRITICAL': 'red,bg_white'}


def main():
    args = get_params()
    config = Config(args.cfg_path)
    short_name = config['general']['short_name']
    config_logging(args.debug, args.logs_dir, short_name)
    micro_mordred(config, args.backend_sections,
                  args.repos_to_check, args.raw,
                  args.identities_merge,
                  args.enrich,
                  args.panels)


def create_sortinghat_client(config):
    """Create a SortingHat client"""

    conf = config.get_conf()

    sortinghat = conf.get('sortinghat', None)
    if not sortinghat:
        return None

    db_user = sortinghat['user'] if sortinghat else None
    db_password = sortinghat['password'] if sortinghat else None
    db_host = sortinghat['host'] if sortinghat else '127.0.0.1'
    db_path = sortinghat.get('path', None) if sortinghat else None
    db_port = sortinghat.get('port', None) if sortinghat else None
    db_ssl = sortinghat.get('ssl', False) if sortinghat else False
    db_verify_ssl = sortinghat.get('verify_ssl', True) if sortinghat else True
    db_tenant = sortinghat.get('tenant', True) if sortinghat else None

    client = SortingHatClient(host=db_host, port=db_port,
                              path=db_path, ssl=db_ssl,
                              user=db_user, password=db_password,
                              verify_ssl=db_verify_ssl,
                              tenant=db_tenant)
    client.connect()

    return client


def micro_mordred(config, backend_sections, repos_to_check, raw, identities_merge, enrich, panels):
    """Execute the Mordred tasks using the configuration file.

    :param config: Mordred configuration file
    :param backend_sections: the backend sections where the raw/enrich/identities phases will be executed
    :param repos_to_check: process a repository only if it is in this list, or `None` for all repos
    :param raw: if true, it activates the collection of raw data
    :param identities_merge: if true, it activates the identities merging process
    :param enrich: if true, it activates the enrichment of the raw data
    :param panels: if true, it activates the upload of all panels declared in the configuration file
    """

    if raw:
        for backend in backend_sections:
            get_raw(config, backend, repos_to_check)

    if identities_merge or enrich:
        sortinghat_client = create_sortinghat_client(config)
    else:
        sortinghat_client = None

    if identities_merge:
        get_identities_merge(config, sortinghat_client)

    if enrich:
        for backend in backend_sections:
            get_enrich(config, sortinghat_client, backend, repos_to_check)

    if panels:
        get_panels(config)


def get_raw(config, backend_section, repos_to_check=None):
    """Execute the raw phase for a given backend section

    Repos are only checked if they are in BOTH `repos_to_check` and the `projects.json`

    :param config: a Mordred config object
    :param backend_section: the backend section where the raw phase is executed
    :param repos_to_check: A list of repo URLs to check, or None to check all repos
    """

    task = TaskRawDataCollection(config, backend_section=backend_section, allowed_repos=repos_to_check)
    TaskProjects(config).execute()
    try:
        task.execute()
        logging.info("Loading raw data finished!")
    except Exception as e:
        logging.error(str(e))
        sys.exit(-1)


def get_identities_merge(config, sortinghat_client):
    """Execute the merge identities phase

    :param sortinghat_client: a SortingHat client
    :param config: a Mordred config object
    """
    TaskProjects(config).execute()
    task = TaskIdentitiesMerge(config, sortinghat_client)
    task.execute()
    logging.info("Merging identities finished!")


def get_enrich(config, sortinghat_client, backend_section, repos_to_check=None):
    """Execute the enrich phase for a given backend section

    Repos are only checked if they are in BOTH `repos_to_check` and the `projects.json`

    :param config: a Mordred config object
    :param sortinghat_client: a SortingHat client
    :param backend_section: the backend section where the enrich phase is executed
    :param repos_to_check: A list of repo URLs to check, or None to check all repos
    """

    TaskProjects(config).execute()
    task = TaskEnrich(config, sortinghat_client, backend_section=backend_section, allowed_repos=repos_to_check)
    try:
        task.execute()
        logging.info("Loading enriched data finished!")
    except Exception as e:
        logging.error(str(e))
        sys.exit(-1)


def get_panels(config):
    """Execute the panels phase

    :param config: a Mordred config object
    """

    task = TaskPanels(config)
    task.execute()

    task = TaskPanelsMenu(config)
    task.execute()

    logging.info("Panels creation finished!")


def config_logging(debug, logs_dir, short_name):
    """Config logging level output output"""

    if debug:
        fmt = f"[%(asctime)s - {short_name} - %(name)s - %(levelname)s] - %(message)s"
        logging_mode = logging.DEBUG
    else:
        fmt = f"%(asctime)s - {short_name} - %(message)s"
        logging_mode = logging.INFO

    # Setting the color scheme and level into the root logger
    logging.basicConfig(level=logging_mode)
    stream = logging.root.handlers[0]
    formatter = colorlog.ColoredFormatter(fmt=COLOR_LOG_FORMAT_SUFFIX + fmt,
                                          log_colors=LOG_COLORS)
    stream.setFormatter(formatter)

    # Creating a file handler and adding it to root
    if logs_dir:
        fh_filepath = os.path.join(logs_dir, 'all.log')
        fh = logging.FileHandler(fh_filepath, mode='w')
        fh.setLevel(logging_mode)
        formatter = logging.Formatter(fmt)
        fh.setFormatter(formatter)
        logging.getLogger().addHandler(fh)

    # ES logger is set to INFO since, it produces a really verbose output if set to DEBUG
    logging.getLogger('elasticsearch').setLevel(logging.INFO)

    # Show if debug mode is activated
    if debug:
        logging.debug("Debug mode activated")


def get_params_parser():
    """Parse command line arguments"""

    parser = argparse.ArgumentParser(add_help=False)

    parser.add_argument('-g', '--debug', dest='debug',
                        action='store_true',
                        help=argparse.SUPPRESS)
    parser.add_argument("--raw", action='store_true', dest='raw',
                        help="Activate raw task")
    parser.add_argument("--enrich", action='store_true', dest='enrich',
                        help="Activate enrich task")
    parser.add_argument("--identities-merge", action='store_true', dest='identities_merge',
                        help="Activate merge identities task")
    parser.add_argument("--panels", action='store_true', dest='panels',
                        help="Activate panels task")

    parser.add_argument("--cfg", dest='cfg_path',
                        help="Configuration file path")
    parser.add_argument("--backends", dest='backend_sections', default=[],
                        nargs='*', help="Backend sections to execute")
    parser.add_argument("--repos", dest='repos_to_check', default=None,
                        nargs='*', help="Limit which repositories are processed (list of URLs)")
    parser.add_argument("--logs-dir", dest='logs_dir', default='', help='Logs Directory')

    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)

    return parser


def get_params():
    """Get params to execute micro-mordred"""

    parser = get_params_parser()
    args = parser.parse_args()

    tasks = [args.raw, args.enrich, args.identities_merge, args.panels]

    if not any(tasks):
        print("No tasks enabled")
        sys.exit(1)

    if args.cfg_path is None:
        print("Config file path not provided")
        sys.exit(1)

    return args


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Alvaro del Castillo <acs@bitergia.com>
#     Quan Zhou <quan@bitergia.com>
#

import json
import logging

from threading import Lock

import requests

from copy import deepcopy

from sirmordred.task import Task

logger = logging.getLogger(__name__)


class TaskProjects(Task):
    """ Task to manage the projects config """

    GLOBAL_PROJECT = 'unknown'  # project to download and enrich full sites
    __projects = {}  # static projects data dict
    projects_lock = Lock()

    def is_backend_task(self):
        return False

    @classmethod
    def get_projects(cls):
        with cls.projects_lock:
            # Return a deepcopy so it is not changed
            return deepcopy(cls.__projects)

    @classmethod
    def set_projects(cls, projects):
        with cls.projects_lock:
            old_projects_hash = hash(json.dumps(cls.__projects, sort_keys=True))
            new_projects_hash = hash(json.dumps(projects, sort_keys=True))

            if old_projects_hash != new_projects_hash:
                logger.debug("Projects file has changed")

            cls.__projects = projects

    @classmethod
    def get_repos_by_backend_section(cls, backend_section, raw=True):
        """ return list with the repositories for a backend_section """
        repos = []
        projects = TaskProjects.get_projects()

        for pro in projects:
            if backend_section in projects[pro]:
                # if the projects.json doesn't contain the `unknown` project, add the repos in the bck section
                if cls.GLOBAL_PROJECT not in projects:
                    repos += projects[pro][backend_section]
                else:
                    # if the projects.json contains the `unknown` project
                    # in the case of the collection phase
                    if raw:
                        # if the current project is not `unknown`
                        if pro != cls.GLOBAL_PROJECT:
                            # if the bck section is not in the `unknown` project, add the repos in the bck section
                            if backend_section not in projects[cls.GLOBAL_PROJECT]:
                                repos += projects[pro][backend_section]
                            # if the backend section is in the `unknown` project,
                            # add the repo in the bck section under `unknown`
                            elif backend_section in projects[pro] and backend_section in projects[cls.GLOBAL_PROJECT]:
                                repos += projects[cls.GLOBAL_PROJECT][backend_section]
                        # if the current project is `unknown`
                        else:
                            # if the backend section is only in the `unknown` project,
                            # add the repo in the bck section under `unknown`
                            not_in_unknown = [projects[pro] for pro in projects if pro != cls.GLOBAL_PROJECT][0]
                            if backend_section not in not_in_unknown:
                                repos += projects[cls.GLOBAL_PROJECT][backend_section]
                    # in the case of the enrichment phase
                    else:
                        # if the current project is not `unknown`
                        if pro != cls.GLOBAL_PROJECT:
                            # if the bck section is not in the `unknown` project, add the repos in the bck section
                            if backend_section not in projects[cls.GLOBAL_PROJECT]:
                                repos += projects[pro][backend_section]
                            # if the backend section is in the `unknown` project, add the repos in the bck section
                            elif backend_section in projects[pro] and backend_section in projects[cls.GLOBAL_PROJECT]:
                                repos += projects[pro][backend_section]
                        # if the current project is `unknown`
                        else:
                            # if the backend section is only in the `unknown` project,
                            # add the repo in the bck section under `unknown`
                            not_in_unknown_prj = [projects[prj] for prj in projects if prj != cls.GLOBAL_PROJECT]
                            not_in_unknown_sections = list(set([section for prj in not_in_unknown_prj
                                                                for section in list(prj.keys())]))
                            if backend_section not in not_in_unknown_sections:
                                repos += projects[pro][backend_section]

        logger.debug("List of repos for %s: %s (raw=%s)", backend_section, repos, raw)

        # avoid duplicated repos
        repos = list(set(repos))
        repos.sort()

        return repos

    def execute(self):
        config = self.conf

        if config['projects']['projects_url']:
            projects = self.__get_projects_from_url()
        else:
            projects_file = config['projects']['projects_file']
            logger.info("Reading projects data from  %s ", projects_file)
            with open(projects_file, 'r') as fprojects:
                projects = json.load(fprojects)

        TaskProjects.set_projects(projects)

    def __get_projects_from_url(self):
        config = self.conf
        projects_url = config['projects']['projects_url']
        projects_file = config['projects']['projects_file']

        logger.info("Getting projects file from URL: %s ", projects_url)
        res = requests.get(projects_url)
        res.raise_for_status()
        projects = res.json()
        with open(projects_file, "w") as fprojects:
            json.dump(projects, fprojects, indent=True)

        return projects



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Valerio Cosentino <valcos@bitergia.com>
#     Miguel Ángel Fernández <mafesan@bitergia.com>

import sys
import tempfile
import unittest

# Hack to make sure that tests import the right packages
# due to setuptools behaviour
sys.path.insert(0, '..')

from sirmordred.config import (Config, logger)

CONF_FULL = 'test.cfg'
CONF_SLIM = 'test_studies.cfg'
CONF_WRONG = 'test_wrong.cfg'


class TestConfig(unittest.TestCase):
    """Config tests"""

    def test_init(self):
        """Test whether attributes are initializated"""

        config = Config(CONF_FULL)

        self.assertIsNotNone(config.conf)
        self.assertIsNone(config.raw_conf)
        self.assertEqual(config.conf_list, [CONF_FULL])
        self.assertEqual(len(config.conf.keys()), 47)

    def test_init_studies(self):
        """Test whether studies' attributes are initialized"""

        config = Config(CONF_SLIM)

        self.assertIsNotNone(config.conf)
        self.assertIsNone(config.raw_conf)
        self.assertEqual(config.conf_list, [CONF_SLIM])

        top_sections = config.conf.keys()
        demography_params = config.conf['enrich_demography:git'].keys()
        enrich_areas_of_code_params = config.conf['enrich_areas_of_code:git'].keys()
        enrich_onion_git_params = config.conf['enrich_onion:git'].keys()
        enrich_extra_data_git_params = config.conf['enrich_extra_data:git'].keys()
        enrich_onion_github_params = config.conf['enrich_onion:github'].keys()
        enrich_onion_gerrit_params = config.conf['enrich_onion:gerrit'].keys()
        enrich_demography_gerrit_params = config.conf['enrich_demography:gerrit'].keys()
        enrich_demography_contribution_gerrit_params = config.conf['enrich_demography_contribution:gerrit'].keys()

        self.assertIn('general', top_sections)
        self.assertIn('projects', top_sections)
        self.assertIn('es_collection', top_sections)
        self.assertIn('es_enrichment', top_sections)
        self.assertIn('sortinghat', top_sections)
        self.assertIn('panels', top_sections)
        self.assertIn('phases', top_sections)

        self.assertIn('git', top_sections)
        self.assertIn('enrich_demography:git', top_sections)
        self.assertIn('date_field', demography_params)
        self.assertIn('author_field', demography_params)

        self.assertIn('enrich_areas_of_code:git', top_sections)
        self.assertIn('in_index', enrich_areas_of_code_params)
        self.assertIn('out_index', enrich_areas_of_code_params)
        self.assertIn('sort_on_field', enrich_areas_of_code_params)
        self.assertIn('no_incremental', enrich_areas_of_code_params)

        self.assertIn('enrich_onion:git', top_sections)
        self.assertIn('in_index', enrich_onion_git_params)
        self.assertIn('out_index', enrich_onion_git_params)
        self.assertIn('data_source', enrich_onion_git_params)
        self.assertIn('contribs_field', enrich_onion_git_params)
        self.assertIn('timeframe_field', enrich_onion_git_params)
        self.assertIn('sort_on_field', enrich_onion_git_params)
        self.assertIn('no_incremental', enrich_onion_git_params)
        self.assertIn('seconds', enrich_onion_git_params)

        self.assertIn('enrich_extra_data:git', top_sections)
        self.assertIn('json_url', enrich_extra_data_git_params)
        self.assertIn('target_index', enrich_extra_data_git_params)

        self.assertIn('github:issue', top_sections)
        self.assertIn('github:pull', top_sections)
        self.assertIn('enrich_onion:github', top_sections)
        self.assertIn('in_index_iss', enrich_onion_github_params)
        self.assertIn('in_index_prs', enrich_onion_github_params)
        self.assertIn('out_index_iss', enrich_onion_github_params)
        self.assertIn('in_index_prs', enrich_onion_github_params)
        self.assertIn('data_source_iss', enrich_onion_github_params)
        self.assertIn('data_source_prs', enrich_onion_github_params)
        self.assertIn('contribs_field', enrich_onion_github_params)
        self.assertIn('timeframe_field', enrich_onion_github_params)
        self.assertIn('sort_on_field', enrich_onion_github_params)
        self.assertIn('no_incremental', enrich_onion_github_params)
        self.assertIn('seconds', enrich_onion_github_params)

        self.assertIn('gerrit', top_sections)
        self.assertIn('enrich_demography_contribution:gerrit', top_sections)
        self.assertIn('date_field', enrich_demography_contribution_gerrit_params)
        self.assertIn('author_field', enrich_demography_contribution_gerrit_params)

        self.assertIn('gerrit', top_sections)
        self.assertIn('enrich_demography:gerrit', top_sections)
        self.assertIn('date_field', enrich_demography_gerrit_params)
        self.assertIn('author_field', enrich_demography_gerrit_params)

        self.assertIn('enrich_onion:gerrit', top_sections)
        self.assertIn('in_index', enrich_onion_gerrit_params)
        self.assertIn('out_index', enrich_onion_gerrit_params)
        self.assertIn('data_source', enrich_onion_gerrit_params)
        self.assertIn('contribs_field', enrich_onion_gerrit_params)
        self.assertIn('timeframe_field', enrich_onion_gerrit_params)
        self.assertIn('sort_on_field', enrich_onion_gerrit_params)
        self.assertIn('no_incremental', enrich_onion_gerrit_params)
        self.assertIn('seconds', enrich_onion_gerrit_params)

        self.assertIn('githubql', top_sections)
        self.assertIn('enrich_reference_analysis', top_sections)

    def test_create_config_file(self):
        """Test whether a config file is correctly created"""

        tmp_path = tempfile.mktemp(prefix='mordred_')

        config = Config(CONF_FULL)
        config.create_config_file(tmp_path)
        copied_config = Config(tmp_path)
        # TODO create_config_file produces a config different from the original one
        # self.assertDictEqual(config.conf, copied_config.conf)

    def test_check_config(self):
        """Test whether the config is properly checked"""

        with self.assertRaises(Exception):
            Config(CONF_WRONG)

    def test_get_data_sources(self):
        """Test whether all data sources are properly retrieved"""

        config = Config(CONF_FULL)

        expected = ['askbot', 'bugzilla', 'bugzillarest', 'confluence', 'discourse', 'dockerhub', 'functest',
                    'gerrit', 'git', 'gitlab', 'github', 'google_hits', 'groupsio', 'hyperkitty', 'jenkins', 'jira',
                    'mbox', 'meetup', 'mediawiki', 'mozillaclub', 'nntp', 'phabricator', 'pipermail', 'puppetforge',
                    'redmine', 'remo', 'rss', 'stackexchange', 'slack', 'supybot', 'telegram', 'twitter']
        data_sources = config.get_data_sources()

        self.assertEqual(len(data_sources), len(expected))
        self.assertEqual(data_sources.sort(), expected.sort())

    def test_set_param(self):
        """Test whether a param is correctly modified"""

        config = Config(CONF_FULL)

        self.assertFalse(config.conf['twitter']['collect'])
        config.set_param("twitter", "collect", "true")
        self.assertTrue(config.conf['twitter']['collect'])

    def test_set_param_not_found(self):
        """Test whether an error is logged if a param does not exist"""

        config = Config(CONF_FULL)

        with self.assertLogs(logger, level='ERROR') as cm:
            config.set_param("twitter", "acme", "true")
            self.assertEqual(cm.output[-1], 'ERROR:sirmordred.config:Config section twitter and param acme not exists')

    def test_backend_composition_by_get_backend_section(self):
        """Test the ability to parameterize backends as in the docs for get_backend_section"""

        config = Config(CONF_FULL)
        config.conf = {
            'backend:param1': {
                'shared_param': 'value 1',
                'unique_to_1': 'value 2',
            },
            'backend:param2': {
                'shared_param': 'value 3',
                'unique_to_2': 'value 4',
            },
            'backend:param1:param2': {
                'param_combo': 'value 5',
            }
        }

        self.assertEqual(config.get_backend_section('backend'), dict())
        self.assertEqual(config.get_backend_section('backend', 'param1'), {
            'shared_param': 'value 1',
            'unique_to_1': 'value 2',
        })
        self.assertEqual(config.get_backend_section('backend', 'param2'), {
            'shared_param': 'value 3',
            'unique_to_2': 'value 4',
        })
        self.assertEqual(config.get_backend_section('backend', 'param2', 'param1'), {
            'shared_param': 'value 1',  # Param1 takes priority
            'unique_to_1': 'value 2',
            'unique_to_2': 'value 4',
        })
        self.assertEqual(config.get_backend_section('backend', 'param1', 'param2'), {
            'shared_param': 'value 3',  # Param2 takes priority
            'unique_to_1': 'value 2',
            'unique_to_2': 'value 4',
            'param_combo': 'value 5',  # This shows up because its an exact match
        })

        config.conf['backend'] = {
            'unique_to_base': 'value 6',
            'shared_param': 'value 7',
        }

        self.assertEqual(config.get_backend_section('backend'), {
            'unique_to_base': 'value 6',
            'shared_param': 'value 7',
        })
        self.assertEqual(config.get_backend_section('backend', 'nonexistant'), {
            'unique_to_base': 'value 6',
            'shared_param': 'value 7',
        })
        self.assertEqual(config.get_backend_section('backend', 'param1'), {
            'unique_to_base': 'value 6',
            'unique_to_1': 'value 2',
            'shared_param': 'value 1',
        })

    def test_backend_composition_by_get_item(self):
        """Test the ability to parameterize backends as in the docs for get_backend_section"""

        # This code is p much copy-pasted from the above test case, save the way that the
        # backends are accessed

        config = Config(CONF_FULL)
        config.conf = {
            'backend:param1': {
                'shared_param': 'value 1',
                'unique_to_1': 'value 2',
            },
            'backend:param2': {
                'shared_param': 'value 3',
                'unique_to_2': 'value 4',
            },
            'backend:param1:param2': {
                'param_combo': 'value 5',
            }
        }

        self.assertEqual(config['backend'], dict())
        self.assertEqual(config['backend:param1'], {
            'shared_param': 'value 1',
            'unique_to_1': 'value 2',
        })
        self.assertEqual(config['backend:param2'], {
            'shared_param': 'value 3',
            'unique_to_2': 'value 4',
        })
        self.assertEqual(config['backend:param2:param1'], {
            'shared_param': 'value 1',  # Param1 takes priority
            'unique_to_1': 'value 2',
            'unique_to_2': 'value 4',
        })
        self.assertEqual(config['backend:param1:param2'], {
            'shared_param': 'value 3',  # Param2 takes priority
            'unique_to_1': 'value 2',
            'unique_to_2': 'value 4',
            'param_combo': 'value 5',  # This shows up because its an exact match
        })

        config.conf['backend'] = {
            'unique_to_base': 'value 6',
            'shared_param': 'value 7',
        }

        self.assertEqual(config['backend'], {
            'unique_to_base': 'value 6',
            'shared_param': 'value 7',
        })
        self.assertEqual(config['backend:nonexistant'], {
            'unique_to_base': 'value 6',
            'shared_param': 'value 7',
        })
        self.assertEqual(config['backend:param1'], {
            'unique_to_base': 'value 6',
            'unique_to_1': 'value 2',
            'shared_param': 'value 1',
        })

    def test_contains(self):

        config = Config(CONF_FULL)
        config.conf = {
            'backend:param1': {
                'shared_param': 'value 1',
                'unique_to_1': 'value 2',
            },
            'backend:param2': {
                'shared_param': 'value 3',
                'unique_to_2': 'value 4',
            },
            'backend:param1:param2': {
                'param_combo': 'value 5',
            }
        }

        # Directly contained
        self.assertIn('backend:param1', config)
        self.assertIn('backend:param2', config)
        self.assertIn('backend:param1:param2', config)

        # Implicitly contained
        self.assertIn('backend:param2:param1', config)
        self.assertIn('backend:fake:param1', config)
        self.assertIn('backend:param1:fake', config)

        # Not contained
        self.assertNotIn('backend', config)
        self.assertNotIn('backend:fake', config)
        self.assertNotIn('fake-backend:param1', config)

        # Set the previously missing backend param
        config.conf['backend'] = {
            'arbitrary_data': 'idc',
        }

        # Contained after backend specified
        self.assertIn('backend', config)
        self.assertIn('backend:fake', config)
        self.assertIn('backend:param1', config)

        # Still not contained
        self.assertNotIn('fake-backend:param1', config)
        self.assertNotIn('fake-backend', config)

    def test_get(self):

        config = Config(CONF_FULL)
        config.conf = {
            'backend:param1': {
                'dummy': 'data',
            },
        }

        self.assertEqual(config.get('backend'), None)
        self.assertEqual(config.get('backend', 'hiya!'), 'hiya!')
        self.assertEqual(config.get('backend:param1'), config['backend:param1'])
        self.assertEqual(config.get('backend:param1', 'inspecific string'), config['backend:param1'])


if __name__ == "__main__":
    # logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(message)s')
    unittest.main(warnings='ignore')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Alvaro del Castillo <acs@bitergia.com>

import sys
import unittest

import httpretty

# Hack to make sure that tests import the right packages
# due to setuptools behaviour
sys.path.insert(0, '..')

from sirmordred.config import Config
from sirmordred.task_projects import TaskProjects


CONF_FILE = 'test.cfg'
PROJECTS_URL = 'http://localhost/projects.json'
URL_PROJECTS_FILE = 'data/url-projects.json'
URL_PROJECTS_MAIN = 'grimoire'

CONF_FILE_UNKNOWN = 'test-repos.cfg'


def read_file(filename, mode='r'):
    with open(filename, mode) as f:
        content = f.read()
    return content


def setup_http_server():
    url_projects = read_file(URL_PROJECTS_FILE)

    http_requests = []

    def request_callback(method, uri, headers):
        last_request = httpretty.last_request()

        body = url_projects
        http_requests.append(last_request)

        return 200, headers, body

    httpretty.register_uri(httpretty.GET,
                           PROJECTS_URL,
                           responses=[
                               httpretty.Response(body=request_callback)
                           ])

    return http_requests


class TestTaskProjects(unittest.TestCase):
    """Task tests"""

    def test_initialization(self):
        """Test whether attributes are initializated"""

        config = Config(CONF_FILE)
        task = TaskProjects(config)

        self.assertEqual(task.config, config)

    def test_get_repos_by_backend_sections_unknown(self):
        """Test whether the repos of each section are properly loaded when the unknown section is present"""

        config = Config(CONF_FILE_UNKNOWN)
        task = TaskProjects(config)
        self.assertEqual(task.execute(), None)

        # repos not in unknown
        expected_list = ["https://github.com/chaoss/grimoirelab-perceval"]

        repos = task.get_repos_by_backend_section("git")
        self.assertListEqual(repos, expected_list)

        repos = task.get_repos_by_backend_section("git", raw=False)
        self.assertListEqual(repos, expected_list)

        # repos only in unknown
        expected_list = ["https://bugzilla.mozilla.org"]

        repos = task.get_repos_by_backend_section("bugzillarest")
        self.assertListEqual(repos, expected_list)

        repos = task.get_repos_by_backend_section("bugzillarest", raw=False)
        self.assertListEqual(repos, expected_list)

        # repos in unknown and other section
        expected_list = ["gerrit.onosproject.org"]

        repos = task.get_repos_by_backend_section("gerrit:onos")
        self.assertListEqual(repos, expected_list)

        expected_list = [
            "gerrit.onosproject.org --filter-raw=data.project:OnosSystemTest",
            "gerrit.onosproject.org --filter-raw=data.project:OnosSystemTestJenkins",
            "gerrit.onosproject.org --filter-raw=data.project:cord-openwrt",
            "gerrit.onosproject.org --filter-raw=data.project:fabric-control",
            "gerrit.onosproject.org --filter-raw=data.project:manifest"
        ]

        repos = task.get_repos_by_backend_section("gerrit:onos", raw=False)
        repos.sort()
        expected_list.sort()
        self.assertListEqual(repos, expected_list)

    def test_get_repos_by_backend_section(self):
        """Test whether the repos of each section are properly loaded"""

        config = Config(CONF_FILE)
        task = TaskProjects(config)
        self.assertEqual(task.execute(), None)

        config.conf.keys()
        backend_sections = list(set([sect for sect in config.conf.keys()
                                     for backend_section in Config.get_backend_sections()
                                     if sect and sect.startswith(backend_section)]))
        backend_sections.sort()
        backend = backend_sections[0]

        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'askbot')
        self.assertEqual(repos, ['https://ask.puppet.com'])

        backend = backend_sections[1]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'bugzilla')
        self.assertEqual(repos, ['https://bugs.eclipse.org/bugs/'])

        backend = backend_sections[2]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'bugzillarest')
        self.assertEqual(repos, ['https://bugzilla.mozilla.org'])

        backend = backend_sections[3]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'confluence')
        self.assertEqual(repos, ['https://wiki.open-o.org/'])

        backend = backend_sections[4]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'discourse')
        self.assertEqual(repos, ['https://foro.mozilla-hispano.org/'])

        backend = backend_sections[5]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'dockerhub')
        self.assertEqual(repos, ['bitergia kibiter'])

        backend = backend_sections[6]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'functest')
        self.assertEqual(repos, ['http://testresults.opnfv.org/test/'])

        backend = backend_sections[7]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'gerrit')
        self.assertEqual(repos, ['review.openstack.org'])

        backend = backend_sections[8]
        repos = task.get_repos_by_backend_section(backend)
        repos.sort()
        expected_list = [
            "https://github.com/VizGrimoire/GrimoireLib "
            "--filter-raw-prefix=data.files.file:grimoirelib_alch,data.files.file:README.md",
            "https://github.com/MetricsGrimoire/CMetrics"]
        expected_list.sort()
        self.assertEqual(backend, 'git')
        self.assertEqual(repos, expected_list)

        backend = backend_sections[9]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'github')
        self.assertEqual(repos, ['https://github.com/grimoirelab/perceval'])

        backend = backend_sections[10]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'github:pull')
        self.assertEqual(repos, ['https://github.com/grimoirelab/perceval'])

        backend = backend_sections[11]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'gitlab')
        self.assertEqual(repos, ['https://gitlab.com/inkscape/inkscape-web'])

        backend = backend_sections[12]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'google_hits')
        self.assertEqual(repos, ['bitergia grimoirelab'])

        backend = backend_sections[13]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'hyperkitty')
        self.assertEqual(repos,
                         ['https://lists.mailman3.org/archives/list/mailman-users@mailman3.org'])

        backend = backend_sections[14]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'jenkins')
        self.assertEqual(repos, ['https://build.opnfv.org/ci'])

        backend = backend_sections[15]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'jira')
        self.assertEqual(repos, ['https://jira.opnfv.org'])

        backend = backend_sections[16]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'mattermost')
        self.assertEqual(repos, ['https://chat.openshift.io 8j366ft5affy3p36987pcugaoa'])

        backend = backend_sections[17]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'mattermost:group1')
        self.assertEqual(repos, ['https://chat.openshift.io 8j366ft5affy3p36987cip'])

        backend = backend_sections[18]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'mattermost:group2')
        self.assertEqual(repos, ['https://chat.openshift.io 8j366ft5affy3p36987ciop'])

        backend = backend_sections[19]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'mbox')
        self.assertEqual(repos, ['metrics-grimoire ~/.perceval/mbox'])

        backend = backend_sections[20]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'mediawiki')
        self.assertEqual(repos, ['https://wiki.mozilla.org'])

        backend = backend_sections[21]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'meetup')
        self.assertEqual(repos, ['South-East-Puppet-User-Group'])

        backend = backend_sections[22]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'mozillaclub')
        self.assertEqual(repos,
                         ['https://spreadsheets.google.com/feeds/cells/'
                          '1QHl2bjBhMslyFzR5XXPzMLdzzx7oeSKTbgR5PM8qp64/ohaibtm/public/values?alt=json'])

        backend = backend_sections[23]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'nntp')
        self.assertEqual(repos, ['news.mozilla.org mozilla.dev.project-link'])

        backend = backend_sections[24]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'phabricator')
        self.assertEqual(repos, ['https://phabricator.wikimedia.org'])

        backend = backend_sections[25]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'pipermail')
        self.assertEqual(repos, ['https://mail.gnome.org/archives/libart-hackers/'])

        backend = backend_sections[26]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'puppetforge')
        self.assertEqual(repos, [''])

        backend = backend_sections[27]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'redmine')
        self.assertEqual(repos, ['http://tracker.ceph.com/'])

        backend = backend_sections[28]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'remo')
        self.assertEqual(repos, ['https://reps.mozilla.org'])

        backend = backend_sections[29]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'remo:activities')
        self.assertEqual(repos, ['https://reps.mozilla.org'])

        backend = backend_sections[30]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'rss')
        self.assertEqual(repos, ['https://blog.bitergia.com/feed/'])

        backend = backend_sections[31]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'slack')
        self.assertEqual(repos, ['C7LSGB0AU'])

        backend = backend_sections[32]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'stackexchange')

        repos.sort()
        expected_list = [
            "https://stackoverflow.com/questions/tagged/ovirt",
            "https://stackoverflow.com/questions/tagged/rdo",
            "https://stackoverflow.com/questions/tagged/kibana"
        ]
        expected_list.sort()
        self.assertEqual(repos, expected_list)

        backend = backend_sections[33]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'supybot')
        self.assertEqual(repos,
                         ['openshift ~/.perceval/irc/percevalbot/logs/ChannelLogger/freenode/#openshift/'])

        backend = backend_sections[34]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'telegram')
        self.assertEqual(repos, ['Mozilla_analytics'])

        backend = backend_sections[35]
        repos = task.get_repos_by_backend_section(backend)
        self.assertEqual(backend, 'twitter')
        self.assertEqual(repos, ['bitergia'])

    def test_run(self):
        """Test whether the Task could be run"""
        config = Config(CONF_FILE)
        task = TaskProjects(config)
        self.assertEqual(task.execute(), None)
        self.assertEqual(len(task.get_projects().keys()), 1)

    def test__get_projects_from_url(self):
        """Test downloading projects from an URL """
        setup_http_server()

        projects_url = 'http://localhost/projects.json'
        config = Config(CONF_FILE)
        config.set_param('projects', 'projects_url', projects_url)
        task = TaskProjects(config)

        httpretty.enable(allow_net_connect=True)
        self.assertEqual(task.execute(), None)

        projects = task.get_projects()
        self.assertTrue(URL_PROJECTS_MAIN in projects)
        httpretty.disable()


if __name__ == "__main__":
    # logging.basicConfig(level=logging.DEBUG, format='%(asctime)s %(message)s')
    unittest.main(warnings='ignore')



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Luis Cañas-Díaz <lcanas@bitergia.com>
#     Alvaro del Castillo <acs@bitergia.com>
#

import copy
import json
import logging
import operator
import requests
import yaml

import panels
from grimoirelab_toolkit.uris import urijoin

from kidash.kidash import import_dashboard, get_dashboard_name, check_kibana_index
from sirmordred.task import Task

logger = logging.getLogger(__name__)

# Header mandatory in ElasticSearch 6
ES6_HEADER = {"Content-Type": "application/json"}
KIBANA_SETTINGS_URL = '/api/kibana/settings'

STRICT_LOADING = "strict"

KAFKA_NAME = 'KIP'
KAFKA_SOURCE = "kafka"
KAFKA_PANEL = "panels/json/kip.json"
KAKFA_IP = "panels/json/kafka-index-pattern.json"

KAFKA_MENU = {
    'name': KAFKA_NAME,
    'source': KAFKA_SOURCE,
    'icon': 'default.png',
    'index-patterns': [KAKFA_IP],
    'menu': [
        {'name': 'Overview', 'panel': KAFKA_PANEL}
    ]
}

COMMUNITY_NAME = 'Community'
COMMUNITY_SOURCE = 'community'
ONION_PANEL_OVERALL = 'panels/json/onion_overall.json'
ONION_PANEL_PROJECTS = 'panels/json/onion_projects.json'
ONION_PANEL_ORGS = 'panels/json/onion_organizations.json'
DEMOGRAPHICS = 'panels/json/demographics.json'
AFFILIATIONS = 'panels/json/affiliations.json'
CONTRIBUTIONS_GROWTH = 'panels/json/contributors_growth.json'
ENGAGEMENT_BY_CONTRIBUTIONS = 'panels/json/engagement_by_contributions.json'
ORGANIZATIONAL_DIVERSITY = 'panels/json/organizational_diversity.json'
ORGANIZATIONAL_DIVERSITY_BY_DOMAINS = 'panels/json/organizational_diversity_by_domains.json'

ONION_PANEL_OVERALL_IP = 'panels/json/all_onion-index-pattern.json'
ONION_PANEL_PROJECTS_IP = 'panels/json/all_onion-index-pattern.json'
ONION_PANEL_ORGS_IP = 'panels/json/all_onion-index-pattern.json'
DEMOGRAPHICS_IP = 'panels/json/demographics-index-pattern.json'
AFFILIATIONS_IP = 'panels/json/affiliations-index-pattern.json'
ALL_ENRICHED_IP = 'panels/json/all_enriched-index-pattern.json'
ALL_ENRICHED_TICKETS_IP = 'panels/json/all_enriched_tickets-index-pattern.json'

COMMUNITY_MENU = {
    'name': COMMUNITY_NAME,
    'source': COMMUNITY_SOURCE,
    'icon': 'default.png',
    'index-patterns': [
        ONION_PANEL_OVERALL_IP,
        ONION_PANEL_PROJECTS_IP,
        ONION_PANEL_ORGS_IP,
        DEMOGRAPHICS_IP,
        AFFILIATIONS_IP,
        ALL_ENRICHED_IP,
        ALL_ENRICHED_TICKETS_IP
    ],
    'menu': [
        {'name': 'Overall', 'panel': ONION_PANEL_OVERALL},
        {'name': 'Projects', 'panel': ONION_PANEL_PROJECTS},
        {'name': 'Organizations', 'panel': ONION_PANEL_ORGS},
        {'name': 'Demographics', 'panel': DEMOGRAPHICS},
        {'name': 'Affiliations', 'panel': AFFILIATIONS},
        {'name': 'Contributors Growth', 'panel': CONTRIBUTIONS_GROWTH},
        {'name': 'Organizational Diversity', 'panel': ORGANIZATIONAL_DIVERSITY},
        {'name': 'Organizational Diversity by Domains', 'panel': ORGANIZATIONAL_DIVERSITY_BY_DOMAINS},
        {'name': 'Engagement', 'panel': ENGAGEMENT_BY_CONTRIBUTIONS}
    ]
}

GITHUB_COMMENTS = "github-comments"
GITHUB_ISSUE_COMMENTS_PANEL = "panels/json/github2_issues_comments_and_collaboration.json"
GITHUB_ISSUE_COMMENTS_IP = "panels/json/github2_issues-index-pattern.json"
GITHUB_PULL_COMMENTS_PANEL = "panels/json/github2_pull_requests_comments_and_collaboration.json"
GITHUB_PULL_COMMENTS_IP = "panels/json/github2_pull_requests-index-pattern.json"

GITHUB_COMMENTS_MENU = {
    'name': 'GitHub Comments',
    'source': GITHUB_COMMENTS,
    'icon': 'default.png',
    'index-patterns': [GITHUB_ISSUE_COMMENTS_IP, GITHUB_PULL_COMMENTS_IP],
    'menu': [
        {'name': 'Issues', 'panel': GITHUB_ISSUE_COMMENTS_PANEL},
        {'name': 'Pull requests', 'panel': GITHUB_PULL_COMMENTS_PANEL}
    ]
}

GITHUB_EVENTS = "github-events"
GITHUB_CLOSED_EVENTS_PANEL = "panels/json/github_events_closed.json"
GITHUB_LABEL_EVENTS_PANEL = "panels/json/github_events_labels.json"
GITHUB_EVENTS_IP = "panels/json/github_events-index-pattern.json"

GITHUB_EVENTS_MENU = {
    'name': 'GitHub Events',
    'source': GITHUB_EVENTS,
    'icon': 'default.png',
    'index-patterns': [GITHUB_EVENTS_IP],
    'menu': [
        {'name': 'Closed events', 'panel': GITHUB_CLOSED_EVENTS_PANEL},
        {'name': 'Label events', 'panel': GITHUB_LABEL_EVENTS_PANEL}
    ]
}

GITHUB_REPOS = "github-repos"
GITHUB_REPOS_PANEL_OVERALL = "panels/json/github_repositories.json"
GITHUB_REPOS_IP = "panels/json/github_repositories-index-pattern.json"

GITHUB_REPOS_MENU = {
    'name': 'GitHub Repositories',
    'source': GITHUB_REPOS,
    'icon': 'default.png',
    'index-patterns': [GITHUB_REPOS_IP],
    'menu': [
        {'name': 'Overview', 'panel': GITHUB_REPOS_PANEL_OVERALL}
    ]
}

GITLAB_ISSUES = "gitlab-issues"
GITLAB_ISSUES_PANEL_OVERALL = "panels/json/gitlab_issues.json"
GITLAB_ISSUES_PANEL_BACKLOG = "panels/json/gitlab_issues_backlog.json"
GITLAB_ISSUES_PANEL_TIMING = "panels/json/gitlab_issues_timing.json"
GITLAB_ISSUES_PANEL_EFFICIENCY = "panels/json/gitlab_issues_efficiency.json"
GITLAB_ISSUES_IP = "panels/json/gitlab_issues-index-pattern.json"

GITLAB_ISSUES_MENU = {
    'name': 'GitLab Issues',
    'source': GITLAB_ISSUES,
    'icon': 'default.png',
    'index-patterns': [GITLAB_ISSUES_IP],
    'menu': [
        {'name': 'Overview', 'panel': GITLAB_ISSUES_PANEL_OVERALL},
        {'name': 'Backlog', 'panel': GITLAB_ISSUES_PANEL_BACKLOG},
        {'name': 'Timing', 'panel': GITLAB_ISSUES_PANEL_TIMING},
        {'name': 'Efficiency', 'panel': GITLAB_ISSUES_PANEL_EFFICIENCY}
    ]
}

GITLAB_MERGES = "gitlab-merges"
GITLAB_MERGES_PANEL_OVERALL = "panels/json/gitlab_merge_requests.json"
GITLAB_MERGES_PANEL_BACKLOG = "panels/json/gitlab_merge_requests_backlog.json"
GITLAB_MERGES_PANEL_TIMING = "panels/json/gitlab_merge_requests_timing.json"
GITLAB_MERGES_PANEL_EFFICIENCY = "panels/json/gitlab_merge_requests_efficiency.json"
GITLAB_MERGES_IP = "panels/json/gitlab_merge_requests-index-pattern.json"

GITLAB_MERGES_MENU = {
    'name': 'GitLab Merges',
    'source': GITLAB_MERGES,
    'icon': 'default.png',
    'index-patterns': [GITLAB_MERGES_IP],
    'menu': [
        {'name': 'Overview', 'panel': GITLAB_MERGES_PANEL_OVERALL},
        {'name': 'Backlog', 'panel': GITLAB_MERGES_PANEL_BACKLOG},
        {'name': 'Timing', 'panel': GITLAB_MERGES_PANEL_TIMING},
        {'name': 'Efficiency', 'panel': GITLAB_MERGES_PANEL_EFFICIENCY}
    ]
}

MATTERMOST = "mattermost"
MATTERMOST_PANEL = "panels/json/mattermost.json"
MATTERMOST_IP = "panels/json/mattermost-index-pattern.json"

MATTERMOST_MENU = {
    'name': 'Mattermost',
    'source': MATTERMOST,
    'icon': 'default.png',
    'index-patterns': [MATTERMOST_IP],
    'menu': [
        {'name': 'Overview', 'panel': MATTERMOST_PANEL}
    ]
}

MATTERMOST = "mattermost"
MATTERMOST_PANEL = "panels/json/mattermost.json"
MATTERMOST_IP = "panels/json/mattermost-index-pattern.json"

COCOM_NAME = 'Code Complexity'
COCOM_SOURCE = 'code-complexity'
COCOM_PANEL = "panels/json/cocom.json"
COCOM_IP = "panels/json/cocom-index-pattern.json"
COCOM_STUDY_IP = "panels/json/cocom_study-index-pattern.json"

COCOM_MENU = {
    'name': COCOM_NAME,
    'source': COCOM_SOURCE,
    'icon': 'default.png',
    'index-patterns': [
        COCOM_IP,
        COCOM_STUDY_IP
    ],
    'menu': [
        {'name': 'Overview', 'panel': COCOM_PANEL}
    ]
}

COLIC_NAME = 'Code License'
COLIC_SOURCE = 'code-license'
COLIC_PANEL = "panels/json/colic.json"
COLIC_IP = "panels/json/colic-index-pattern.json"
COLIC_STUDY_IP = "panels/json/colic_study-index-pattern.json"

COLIC_MENU = {
    'name': COLIC_NAME,
    'source': COLIC_SOURCE,
    'icon': 'default.png',
    'index-patterns': [
        COLIC_IP,
        COLIC_STUDY_IP
    ],
    'menu': [
        {'name': 'Overview', 'panel': COLIC_PANEL}
    ]
}


def get_sigils_path():
    sigils_path = panels.__file__.replace('panels/__init__.py', '')
    return sigils_path


class TaskPanels(Task):
    """
    Upload all the Kibana dashboards/GrimoireLab panels based on
    enabled data sources AND the menu file. Before uploading the panels
    it also sets up common configuration variables for Kibiter
    """

    # Panels which include several data sources
    panels_multi_ds = ["panels/json/overview.json", "panels/json/data_status.json"]
    # Panels to be uploaded always, no matter the data sources configured
    panels_common = panels_multi_ds + ["panels/json/about.json"]

    def __init__(self, conf, sortinghat_client=None):
        super().__init__(conf, sortinghat_client)
        # Read panels and menu description from yaml file
        with open(self.conf['general']['menu_file'], 'r') as f:
            try:
                self.panels_menu = yaml.load(f, Loader=yaml.SafeLoader)
            except yaml.YAMLError as ex:
                logger.error(ex)
                raise

        # FIXME exceptions raised here are not handled!!

        # Gets the cross set of enabled data sources and data sources
        # available in the menu file. For the result set gets the file
        # names of dashoards and index pattern to be uploaded
        enabled_ds = conf.get_data_sources()
        self.panels = {}

        for ds in self.panels_menu:
            if ds['source'] not in enabled_ds:
                continue
            for entry in ds['menu']:
                if ds['source'] not in self.panels.keys():
                    self.panels[ds['source']] = []
                self.panels[ds['source']].append(entry['panel'])
            if 'index-patterns' in ds:
                for index_pattern in ds['index-patterns']:
                    self.panels[ds['source']].append(index_pattern)

        if self.conf['panels'][COMMUNITY_SOURCE]:
            self.panels[COMMUNITY_SOURCE] = [ONION_PANEL_OVERALL, ONION_PANEL_PROJECTS,
                                             ONION_PANEL_ORGS, DEMOGRAPHICS, AFFILIATIONS,
                                             ONION_PANEL_OVERALL_IP, ONION_PANEL_PROJECTS_IP,
                                             ONION_PANEL_ORGS_IP, DEMOGRAPHICS_IP, AFFILIATIONS_IP,
                                             ENGAGEMENT_BY_CONTRIBUTIONS, CONTRIBUTIONS_GROWTH,
                                             ORGANIZATIONAL_DIVERSITY, ORGANIZATIONAL_DIVERSITY_BY_DOMAINS,
                                             ALL_ENRICHED_IP, ALL_ENRICHED_TICKETS_IP]

        if self.conf['panels'][KAFKA_SOURCE]:
            self.panels[KAFKA_SOURCE] = [KAFKA_PANEL, KAKFA_IP]

        if self.conf['panels'][GITHUB_COMMENTS]:
            self.panels[GITHUB_COMMENTS] = [GITHUB_ISSUE_COMMENTS_PANEL, GITHUB_PULL_COMMENTS_PANEL,
                                            GITHUB_ISSUE_COMMENTS_IP, GITHUB_PULL_COMMENTS_IP]

        if self.conf['panels'][GITHUB_EVENTS]:
            self.panels[GITHUB_EVENTS] = [GITHUB_CLOSED_EVENTS_PANEL, GITHUB_LABEL_EVENTS_PANEL, GITHUB_EVENTS_IP]

        if self.conf['panels'][GITHUB_REPOS]:
            self.panels[GITHUB_REPOS] = [GITHUB_REPOS_PANEL_OVERALL, GITHUB_REPOS_IP]

        if self.conf['panels'][GITLAB_ISSUES]:
            self.panels[GITLAB_ISSUES] = [GITLAB_ISSUES_PANEL_BACKLOG, GITLAB_ISSUES_PANEL_OVERALL,
                                          GITLAB_ISSUES_PANEL_TIMING, GITLAB_ISSUES_PANEL_EFFICIENCY,
                                          GITLAB_ISSUES_IP]

        if self.conf['panels'][GITLAB_MERGES]:
            self.panels[GITLAB_MERGES] = [GITLAB_MERGES_PANEL_BACKLOG, GITLAB_MERGES_PANEL_OVERALL,
                                          GITLAB_MERGES_PANEL_TIMING, GITLAB_MERGES_PANEL_EFFICIENCY,
                                          GITLAB_MERGES_IP]

        if self.conf['panels'][MATTERMOST]:
            self.panels[MATTERMOST] = [MATTERMOST_PANEL, MATTERMOST_IP]

        if self.conf['panels'][COCOM_SOURCE]:
            self.panels[COCOM_SOURCE] = [COCOM_PANEL, COCOM_IP, COCOM_STUDY_IP]

        if self.conf['panels'][COLIC_SOURCE]:
            self.panels[COLIC_SOURCE] = [COLIC_PANEL, COLIC_IP, COLIC_STUDY_IP]

    def is_backend_task(self):
        return False

    def __configure_kibiter_setting(self, endpoint, data_value=None):
        kibana_headers = copy.deepcopy(ES6_HEADER)
        kibana_headers["kbn-xsrf"] = "true"

        kibana_url = self.conf['panels']['kibiter_url'] + KIBANA_SETTINGS_URL
        endpoint_url = kibana_url + '/' + endpoint

        try:
            res = self.grimoire_con.post(endpoint_url, headers=kibana_headers,
                                         data=json.dumps(data_value), verify=False)
            res.raise_for_status()
        except requests.exceptions.HTTPError:
            logger.error("Impossible to set %s: %s", endpoint, str(res.json()))
            return False
        except requests.exceptions.ConnectionError as ex:
            logger.error("Impossible to connect to kibiter %s: %s",
                         self.anonymize_url(self.conf['panels']['kibiter_url']), ex)
            return False

        return True

    def __configure_kibiter_6(self):
        if 'panels' not in self.conf:
            logger.warning("Panels config not availble. Not configuring Kibiter.")
            return False

        # Create .kibana index if not exists
        es_url = self.conf['es_enrichment']['url']
        kibiter_url = self.conf['panels']['kibiter_url']
        check_kibana_index(es_url, kibiter_url)

        # set default index pattern
        kibiter_default_index = self.conf['panels']['kibiter_default_index']
        data_value = {"value": kibiter_default_index}
        defaultIndexFlag = self.__configure_kibiter_setting('defaultIndex',
                                                            data_value=data_value)

        # set default time picker
        kibiter_time_from = self.conf['panels']['kibiter_time_from']
        time_picker = {"from": kibiter_time_from, "to": "now", "mode": "quick"}
        data_value = {"value": json.dumps(time_picker)}
        timePickerFlag = self.__configure_kibiter_setting('timepicker:timeDefaults',
                                                          data_value=data_value)

        if defaultIndexFlag and timePickerFlag:
            logger.info("Kibiter settings configured!")
            return True

        logger.error("Kibiter settings not configured!")
        return False

    def create_dashboard(self, panel_file, data_sources=None, strict=True):
        """Upload a panel to Elasticsearch if it does not exist yet.

        If a list of data sources is specified, upload only those
        elements (visualizations, searches) that match that data source.

        :param panel_file: file name of panel (dashobard) to upload
        :param data_sources: list of data sources
        :param strict: only upload a dashboard if it is newer than the one already existing
        """
        es_enrich = self.conf['es_enrichment']['url']
        kibana_url = self.conf['panels']['kibiter_url']

        mboxes_sources = set(['pipermail', 'hyperkitty', 'groupsio', 'nntp'])
        if data_sources and any(x in data_sources for x in mboxes_sources):
            data_sources = list(data_sources)
            data_sources.append('mbox')
        if data_sources and ('supybot' in data_sources):
            data_sources = list(data_sources)
            data_sources.append('irc')
        if data_sources and 'google_hits' in data_sources:
            data_sources = list(data_sources)
            data_sources.append('googlehits')
        if data_sources and 'stackexchange' in data_sources:
            # stackexchange is called stackoverflow in panels
            data_sources = list(data_sources)
            data_sources.append('stackoverflow')
        if data_sources and 'phabricator' in data_sources:
            data_sources = list(data_sources)
            data_sources.append('maniphest')

        panels_path = get_sigils_path() + panel_file
        try:
            import_dashboard(es_enrich, kibana_url, panels_path, data_sources=data_sources, strict=strict)
        except ValueError:
            logger.error("%s does not include release field. Not loading the panel.", panels_path)
        except RuntimeError:
            logger.error("Can not load the panel %s", panels_path)

    def execute(self):
        # Configure kibiter
        kibiter_major = self.es_version(self.conf['es_enrichment']['url'])
        strict_loading = self.conf['panels'][STRICT_LOADING]

        self.__configure_kibiter_6()

        logger.info("Dashboard panels, visualizations: uploading...")
        # Create the commons panels
        for panel_file in self.panels_common:
            data_sources = None  # for some panels, only the active data sources must be included
            if panel_file in TaskPanels.panels_multi_ds:
                data_sources = self.panels.keys()
            self.create_dashboard(panel_file, data_sources=data_sources, strict=strict_loading)

        # Upload all the Kibana dashboards/GrimoireLab panels based on
        # enabled data sources AND the menu file
        for ds in self.panels:
            for panel_file in self.panels[ds]:
                try:
                    self.create_dashboard(panel_file, strict=strict_loading)
                except Exception as ex:
                    logger.error("%s not correctly uploaded (%s)", panel_file, ex)
        logger.info("Dashboard panels, visualizations: uploaded!")


class TaskPanelsMenu(Task):
    """Create the menu to access the panels"""

    menu_panels_common = {
        "Overview": {
            "title": "Overview",
            "name": "Overview",
            "description": "Overview panel",
            "type": "entry",
            "panel_id": "Overview"
        },
        "Data Status": {
            "title": "Data Status",
            "name": "Data Status",
            "description": "Data Status panel",
            "type": "entry",
            "panel_id": "Data-Status"
        }
    }

    def __init__(self, conf, sortinghat_client=None):
        super().__init__(conf, sortinghat_client)
        # Read panels and menu description from yaml file """
        with open(self.conf['general']['menu_file'], 'r') as f:
            try:
                self.panels_menu = yaml.load(f, Loader=yaml.SafeLoader)
            except yaml.YAMLError as ex:
                logger.error(ex)
                raise

        if self.conf['panels'][GITHUB_COMMENTS]:
            self.panels_menu.append(GITHUB_COMMENTS_MENU)

        if self.conf['panels'][GITHUB_EVENTS]:
            self.panels_menu.append(GITHUB_EVENTS_MENU)

        if self.conf['panels'][GITHUB_REPOS]:
            self.panels_menu.append(GITHUB_REPOS_MENU)

        if self.conf['panels'][GITLAB_ISSUES]:
            self.panels_menu.append(GITLAB_ISSUES_MENU)

        if self.conf['panels'][GITLAB_MERGES]:
            self.panels_menu.append(GITLAB_MERGES_MENU)

        if self.conf['panels'][MATTERMOST]:
            self.panels_menu.append(MATTERMOST_MENU)

        if self.conf['panels'][COMMUNITY_SOURCE]:
            self.panels_menu.append(COMMUNITY_MENU)

        if self.conf['panels'][KAFKA_SOURCE]:
            self.panels_menu.append(KAFKA_MENU)

        if self.conf['panels'][COCOM_SOURCE]:
            self.panels_menu.append(COCOM_MENU)

        if self.conf['panels'][COLIC_SOURCE]:
            self.panels_menu.append(COLIC_MENU)

        # Get the active data sources
        self.data_sources = self.__get_active_data_sources()
        if 'short_name' in self.conf['general']:
            self.project_name = self.conf['general']['short_name']
        else:
            self.project_name = 'GrimoireLab'

    def is_backend_task(self):
        return False

    def __get_active_data_sources(self):
        active_ds = []
        for entry in self.panels_menu:
            ds = entry['source']
            if ds in self.conf or ds in [COMMUNITY_SOURCE, KAFKA_SOURCE, GITLAB_ISSUES, GITLAB_MERGES,
                                         MATTERMOST, GITHUB_COMMENTS, GITHUB_REPOS, GITHUB_EVENTS,
                                         COCOM_SOURCE, COLIC_SOURCE]:
                active_ds.append(ds)
        logger.debug("Active data sources for menu: %s", active_ds)

        return active_ds

    def __upload_title(self, kibiter_major):
        """Upload to Kibiter the title for the dashboard.

        The title is shown on top of the dashboard menu, and is Usually
        the name of the project being dashboarded.
        This is done only for Kibiter 6.x.

        :param kibiter_major: major version of kibiter
        """
        resource = ".kibana/doc/projectname"
        data = {"projectname": {"name": self.project_name}}
        mapping_resource = ".kibana/_mapping/doc"
        mapping = {"dynamic": "true"}

        url = urijoin(self.conf['es_enrichment']['url'], resource)
        mapping_url = urijoin(self.conf['es_enrichment']['url'],
                              mapping_resource)

        logger.debug("Adding mapping for dashboard title")
        res = self.grimoire_con.put(mapping_url, data=json.dumps(mapping),
                                    headers=ES6_HEADER)
        try:
            res.raise_for_status()
        except requests.exceptions.HTTPError:
            logger.error("Couldn't create mapping for dashboard title.")
            logger.error(res.json())

        logger.debug("Uploading dashboard title")
        res = self.grimoire_con.post(url, data=json.dumps(data),
                                     headers=ES6_HEADER)
        try:
            res.raise_for_status()
        except requests.exceptions.HTTPError:
            logger.error("Couldn't create dashboard title.")
            logger.error(res.json())

    def __create_dashboard_menu(self, dash_menu, kibiter_major):
        """Create the menu definition to access the panels in a dashboard.

        :param          menu: dashboard menu to upload
        :param kibiter_major: major version of kibiter
        """
        logger.info("Adding dashboard menu")
        menu_resource = ".kibana/doc/metadashboard"
        mapping_resource = ".kibana/_mapping/doc"
        mapping = {"dynamic": "true"}
        menu = {'metadashboard': dash_menu}
        menu_url = urijoin(self.conf['es_enrichment']['url'],
                           menu_resource)

        mapping_url = urijoin(self.conf['es_enrichment']['url'],
                              mapping_resource)
        logger.debug("Adding mapping for metadashboard")
        res = self.grimoire_con.put(mapping_url, data=json.dumps(mapping),
                                    headers=ES6_HEADER)
        try:
            res.raise_for_status()
        except requests.exceptions.HTTPError:
            logger.error("Couldn't create mapping for Kibiter menu.")
        res = self.grimoire_con.post(menu_url, data=json.dumps(menu),
                                     headers=ES6_HEADER)
        try:
            res.raise_for_status()
        except requests.exceptions.HTTPError:
            logger.error("Couldn't create Kibiter menu.")
            logger.error(res.json())
            raise

    def __remove_dashboard_menu(self, kibiter_major):
        """Remove existing menu for dashboard, if any.

        Usually, we remove the menu before creating a new one.

        :param kibiter_major: major version of kibiter
        """
        logger.info("Removing old dashboard menu, if any")
        metadashboard = ".kibana/doc/metadashboard"
        menu_url = urijoin(self.conf['es_enrichment']['url'], metadashboard)
        self.grimoire_con.delete(menu_url)

    def __get_menu_entries(self, kibiter_major):
        """ Get the menu entries from the panel definition """
        menu_entries = []
        for entry in self.panels_menu:
            if entry['source'] not in self.data_sources:
                continue
            parent_menu_item = {
                'name': entry['name'],
                'title': entry['name'],
                'description': "",
                'type': "menu",
                'dashboards': []
            }
            for subentry in entry['menu']:
                try:
                    panel_path = get_sigils_path() + subentry['panel']
                    dash_name = get_dashboard_name(panel_path)
                except FileNotFoundError:
                    logging.error("Can't open dashboard file %s", subentry['panel'])
                    continue
                # The name for the entry is in self.panels_menu
                child_item = {
                    "name": subentry['name'],
                    "title": subentry['name'],
                    "description": "",
                    "type": "entry",
                    "panel_id": dash_name
                }
                parent_menu_item['dashboards'].append(child_item)
            menu_entries.append(parent_menu_item)

        return menu_entries

    def __get_about_menu(self, contact_link):
        """ Fill About entry with the panel and the contact link """
        about_menu_item = {
            'name': 'About',
            'title': 'About',
            'description': "About menu",
            'type': "menu",
            'dashboards': [
                {
                    "title": "About the dashboard",
                    "name": "About the dashboard",
                    "description": "Panel with information about the usage of the interface, an information about the \
                                    panel itself and acknowledgments",
                    "type": "entry",
                    "panel_id": "About"
                }
            ]
        }

        if contact_link:
            contact_entry = {
                "title": "Contact",
                "name": "Contact",
                "description": "Direct link to the support repository",
                "type": "entry",
                "panel_id": contact_link
            }
            about_menu_item['dashboards'].append(contact_entry)

        return about_menu_item

    def __get_dash_menu(self, kibiter_major, contact_link):
        """Order the dashboard menu"""

        # omenu = OrderedDict()
        omenu = []
        # Start with Overview
        omenu.append(self.menu_panels_common['Overview'])

        # Now the data _getsources
        ds_menu = self.__get_menu_entries(kibiter_major)

        # Remove the kafka and community menus, they will be included at the end
        kafka_menu = None
        community_menu = None
        cocom_menu = None
        colic_menu = None

        found_cocom = [pos for pos, menu in enumerate(ds_menu) if menu['name'] == COCOM_NAME]
        if found_cocom:
            cocom_menu = ds_menu.pop(found_cocom[0])

        found_colic = [pos for pos, menu in enumerate(ds_menu) if menu['name'] == COLIC_NAME]
        if found_colic:
            colic_menu = ds_menu.pop(found_colic[0])

        found_kafka = [pos for pos, menu in enumerate(ds_menu) if menu['name'] == KAFKA_NAME]
        if found_kafka:
            kafka_menu = ds_menu.pop(found_kafka[0])

        found_community = [pos for pos, menu in enumerate(ds_menu) if menu['name'] == COMMUNITY_NAME]
        if found_community:
            community_menu = ds_menu.pop(found_community[0])

        ds_menu.sort(key=operator.itemgetter('name'))
        omenu += ds_menu

        # If kafka and community are present add them before the Data Status and About
        if kafka_menu:
            omenu.append(kafka_menu)

        if cocom_menu:
            omenu.append(cocom_menu)

        if colic_menu:
            omenu.append(colic_menu)

        if community_menu:
            omenu.append(community_menu)

        # At the end Data Status
        omenu.append(self.menu_panels_common['Data Status'])

        # And About
        omenu.append(self.__get_about_menu(contact_link))

        logger.debug("Menu for panels: %s", json.dumps(ds_menu, indent=4))
        return omenu

    def execute(self):
        kibiter_major = self.es_version(self.conf['es_enrichment']['url'])

        logger.info("Dashboard menu: uploading for %s ..." % kibiter_major)
        # Create the panels menu
        menu = self.__get_dash_menu(kibiter_major, self.conf['panels']['contact'])
        # Remove the current menu and create the new one
        self.__upload_title(kibiter_major)
        self.__remove_dashboard_menu(kibiter_major)
        self.__create_dashboard_menu(menu, kibiter_major)
        logger.info("Dashboard menu: uploaded!")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2023 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#       Alvaro del Castillo <acs@bitergia.com>
#       Luis Cañas-Díaz <lcanas@bitergia.com>
#       Quan Zhou <quan@bitergia.com>
#       Miguel Ángel Fernández <mafesan@bitergia.com>
#

import configparser
import logging
from typing import Any, Dict, TypeVar, Union

from sirmordred.task import Task
from grimoire_elk.utils import get_connectors

logger = logging.getLogger(__name__)


MENU_YAML = 'menu.yaml'
ALIASES_JSON = 'aliases.json'
PROJECTS_JSON = 'projects.json'
GLOBAL_DATA_SOURCES = ['bugzilla', 'bugzillarest', 'confluence',
                       'discourse', 'gerrit', 'jenkins', 'jira']


class Config():
    """Class aimed to manage sirmordred configuration"""

    def __init__(self, conf_file, conf_list=[]):
        """Initialize object.

        The object can be initialized with a configuration file,
        and, optionally, with a list of other configuration files.
        If the list of other configuration files exist, it will
        be read, in order, after the configuration file.
        Values set in a file read later will overwrite values set
        in files read earlier. Values not set by any file will
        be set to the default values, when possible.

        :param conf_file; configuration file name
        :param conf_list: list of other configuration files (default: empty)
        """

        self.conf_list = [conf_file] + conf_list
        self.raw_conf = None
        # Build self.conf
        self.__read_conf_files()

    @classmethod
    def backend_section_params(self):
        # Params that must exists in all backends
        params = {
            "enriched_index": {
                "optional": False,
                "default": None,
                "type": str,
                "description": "Index name in which to store the enriched items"
            },
            "raw_index": {
                "optional": False,
                "default": None,
                "type": str,
                "description": "Index name in which to store the raw items"
            },
            "studies": {
                "optional": True,
                "default": [],
                "type": list,
                "description": "List of studies to be executed"
            }
        }

        return params

    @classmethod
    def general_params(cls):
        """ Define all the possible config params """

        params = {}

        # GENERAL CONFIG
        params_general = {
            "general": {
                "min_update_delay": {
                    "optional": True,
                    "default": 60,
                    "type": int,
                    "description": "Short delay between tasks (collect, enrich ...)"
                },
                "update_hour": {
                    "optional": True,
                    "default": None,
                    "type": int,
                    "description": "Start the next execution (collect, enrich ...)"
                },
                "update": {
                    "optional": False,
                    "default": False,
                    "type": bool,
                    "description": "Execute the tasks in loop"
                },
                "short_name": {
                    "optional": False,
                    "default": "Short name",
                    "type": str,
                    "description": "Short name of the project"
                },
                "debug": {
                    "optional": False,
                    "default": True,
                    "type": bool,
                    "description": "Debug mode (logging mainly)"
                },
                "logs_dir": {
                    "optional": True,
                    "default": None,
                    "type": str,
                    "description": "Directory with the logs of sirmordred"
                },
                "bulk_size": {
                    "optional": True,
                    "default": 1000,
                    "type": int,
                    "description": "Number of items to write in Elasticsearch using bulk operations"
                },
                "scroll_size": {
                    "optional": True,
                    "default": 100,
                    "type": int,
                    "description": "Number of items to read from Elasticsearch when scrolling"
                },
                "aliases_file": {
                    "optional": True,
                    "default": ALIASES_JSON,
                    "type": str,
                    "description": "JSON file to define aliases for raw and enriched indexes"
                },
                "menu_file": {
                    "optional": True,
                    "default": MENU_YAML,
                    "type": str,
                    "description": "YAML file to define the menus to be shown in Kibiter"
                },
                "retention_time": {
                    "optional": True,
                    "default": None,
                    "type": int,
                    "description": "The maximum number of minutes wrt the current date to retain the data"
                }
            }
        }
        params_projects = {
            "projects": {
                "projects_file": {
                    "optional": True,
                    "default": PROJECTS_JSON,
                    "type": str,
                    "description": "Projects file path with repositories to be collected group by projects"
                },
                "projects_url": {
                    "optional": True,
                    "default": None,
                    "type": str,
                    "description": "Projects file URL"
                }
            }
        }

        params_phases = {
            "phases": {
                "collection": {
                    "optional": False,
                    "default": True,
                    "type": bool,
                    "description": "Activate collection of items"
                },
                "enrichment": {
                    "optional": False,
                    "default": True,
                    "type": bool,
                    "description": "Activate enrichment of items"
                },
                "identities": {
                    "optional": False,
                    "default": True,
                    "type": bool,
                    "description": "Do the identities tasks"
                },
                "panels": {
                    "optional": False,
                    "default": True,
                    "type": bool,
                    "description": "Load panels, create alias and other tasks related"
                }
            }
        }

        general_config_params = [params_general, params_projects, params_phases]

        for section_params in general_config_params:
            params.update(section_params)

        # Config provided by tasks
        params_collection = {
            "es_collection": {
                "url": {
                    "optional": False,
                    "default": "http://172.17.0.1:9200",
                    "type": str,
                    "description": "Elasticsearch URL"
                }
            }
        }

        params_enrichment = {
            "es_enrichment": {
                "url": {
                    "optional": False,
                    "default": "http://172.17.0.1:9200",
                    "type": str,
                    "description": "Elasticsearch URL"
                },
                "autorefresh": {
                    "optional": True,
                    "default": True,
                    "type": bool,
                    "description": "Execute the autorefresh of identities"
                },
                "autorefresh_interval": {
                    "optional": True,
                    "default": 2,
                    "type": int,
                    "description": "Set time interval (days) for autorefresh identities"
                }
            }
        }

        params_panels = {
            "panels": {
                "strict": {
                    "optional": True,
                    "default": True,
                    "type": bool,
                    "description": "Enable strict panels loading"
                },
                "kibiter_time_from": {
                    "optional": True,
                    "default": "now-90d",
                    "type": str,
                    "description": "Default time interval for Kibiter"
                },
                "kibiter_default_index": {
                    "optional": True,
                    "default": "git",
                    "type": str,
                    "description": "Default index pattern for Kibiter"
                },
                "kibiter_url": {
                    "optional": False,
                    "default": None,
                    "type": str,
                    "description": "Kibiter URL"
                },
                "kibiter_version": {
                    "optional": True,
                    "default": None,
                    "type": str,
                    "description": "Kibiter version"
                },
                "community": {
                    "optional": True,
                    "default": True,
                    "type": bool,
                    "description": "Enable community structure menu"
                },
                "kafka": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "Enable kafka menu"
                },
                "github-comments": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "Enable GitHub comments menu"
                },
                "github-events": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "Enable GitHub events menu"
                },
                "github-repos": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "Enable GitHub repo stats menu"
                },
                "gitlab-issues": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "Enable GitLab issues menu"
                },
                "gitlab-merges": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "Enable GitLab merge requests menu"
                },
                "mattermost": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "Enable Mattermost menu"
                },
                "code-license": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "Enable Code License menu"
                },
                "code-complexity": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "Enable Code Complexity menu"
                },
                "contact": {
                    "optional": True,
                    "default": None,
                    "type": str,
                    "description": "Support repository URL"
                }
            }
        }

        params_sortinghat = {
            "sortinghat": {
                "affiliate": {
                    "optional": False,
                    "default": "True",
                    "type": bool,
                    "description": "Affiliate identities to organizations"
                },
                "unaffiliated_group": {
                    "optional": False,
                    "default": "Unknown",
                    "type": str,
                    "description": "Name of the organization for unaffiliated identities"
                },
                "matching": {
                    "optional": False,
                    "default": ["email"],
                    "type": list,
                    "description": "Algorithm for matching identities in Sortinghat"
                },
                "sleep_for": {
                    "optional": False,
                    "default": 3600,
                    "type": int,
                    "description": "Delay between task identities executions"
                },
                "database": {
                    "optional": False,
                    "default": "sortinghat_db",
                    "type": str,
                    "description": "Name of the Sortinghat database"
                },
                "host": {
                    "optional": False,
                    "default": "mariadb",
                    "type": str,
                    "description": "Host with the Sortinghat database"
                },
                "user": {
                    "optional": False,
                    "default": "root",
                    "type": str,
                    "description": "User to access the Sortinghat database"
                },
                "password": {
                    "optional": False,
                    "default": "",
                    "type": str,
                    "description": "Password to access the Sortinghat database"
                },
                "autoprofile": {
                    "optional": False,
                    "default": ["customer", "git", "github"],
                    "type": list,
                    "description": "Order in which to get the identities information for filling the profile"
                },
                "load_orgs": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "deprecated": "Load organizations in Sortinghat database",
                    "description": ""
                },
                "identities_format": {
                    "optional": True,
                    "default": "sortinghat",
                    "type": str,
                    "description": "Format of the identities data to be loaded"
                },
                "strict_mapping": {
                    "optional": True,
                    "default": True,
                    "type": bool,
                    "description": "rigorous check of values in identities matching "
                                   "(i.e, well formed email addresses)"
                },
                "reset_on_load": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "Unmerge and remove affiliations for all identities on load"
                },
                "orgs_file": {
                    "optional": True,
                    "default": None,
                    "type": str,
                    "description": "File path with the organizations to be loaded in Sortinghat"
                },
                "identities_file": {
                    "optional": True,
                    "default": [],
                    "type": list,
                    "description": "File path with the identities to be loaded in Sortinghat"
                },
                "identities_export_url": {
                    "optional": True,
                    "default": None,
                    "type": str,
                    "description": "URL in which to export the identities in Sortinghat"
                },
                "identities_api_token": {
                    "optional": True,
                    "default": None,
                    "type": str,
                    "description": "API token for remote operation with GitHub and Gitlab"
                },
                "autogender": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "Add gender to the profiles (executes autogender)"
                },
                "path": {
                    "optional": True,
                    "default": None,
                    "type": str,
                    "description": "GraphQL path"
                },
                "port": {
                    "optional": True,
                    "default": None,
                    "type": int,
                    "description": "GraphQL server port"
                },
                "ssl": {
                    "optional": True,
                    "default": False,
                    "type": bool,
                    "description": "GraphQL server use SSL/TSL connection"
                },
                "verify_ssl": {
                    "optional": True,
                    "default": True,
                    "type": bool,
                    "description": "Verify SSL connection to the server"
                },
                "tenant": {
                    "optional": True,
                    "default": None,
                    "type": str,
                    "description": "Tenant name when multi-tenancy is enabled"
                }
            }
        }

        tasks_config_params = [params_collection, params_enrichment, params_panels, params_sortinghat]
        for section_params in tasks_config_params:
            params.update(section_params)

        return params

    @classmethod
    def create_config_file(cls, file_path):
        logger.info("Creating config file in %s", file_path)
        general_sections = cls.general_params()
        backend_sections = cls.get_backend_sections()

        parser = configparser.ConfigParser()

        sections = list(general_sections.keys())
        sections.sort()
        for section_name in sections:
            parser.add_section(section_name)
            section = general_sections[section_name]
            params = list(section.keys())
            params.sort()
            for param in params:
                parser.set(section_name, param, str(section[param]["default"]))

        sections = backend_sections
        sections.sort()
        backend_params = cls.backend_section_params()
        params = list(cls.backend_section_params().keys())
        params.sort()
        for section_name in sections:
            parser.add_section(section_name)
            for param in params:
                if param == "enriched_index":
                    val = section_name
                elif param == "raw_index":
                    val = section_name + "-raw"
                else:
                    val = backend_params[param]['default']
                parser.set(section_name, param, str(val))

        with open(file_path, "w") as f:
            parser.write(f)

    def get_conf(self):
        """
        Return this object.

        This method exists for legacy reasons, and has been depricated.  Please prefer to
        just use this object directly.

        .. deprecated:: 0.2.39
        """
        return self

    def set_param(self, section, param, value):
        """ Change a param in the config """
        if section not in self.conf or param not in self.conf[section]:
            logger.error('Config section %s and param %s not exists', section, param)
        else:
            self.conf[section][param] = value

    @classmethod
    def get_backend_sections(cls):
        # a backend name could include and extra ":<param>"
        # to have several backend entries with different configs
        gelk_backends = list(get_connectors().keys())
        extra_backends = ["apache"]

        return gelk_backends + extra_backends

    @classmethod
    def get_study_sections(cls):
        # a study name could include and extra ":<param>"
        # to have several backend entries with different configs
        studies = []

        connectors = get_connectors()
        for _, backends in connectors.items():
            enrich_backend = backends[2]()
            for study in enrich_backend.studies:
                studies.append(study.__name__)

        return tuple(set(studies))

    def get_active_data_sources(self):
        data_sources = []
        backend_sections = self.get_backend_sections()

        for section in self.conf.keys():
            data_source = section.split(":")[0]
            if data_source in backend_sections:
                data_sources.append(data_source)

        return data_sources

    def get_data_sources(self):
        data_sources = []
        backend_sections = self.get_backend_sections()

        for section in self.conf.keys():
            if section in backend_sections:
                data_sources.append(section)

        return data_sources

    @classmethod
    def check_config(cls, config):
        # First let's check all common sections entries
        check_params = cls.general_params()
        backend_sections = cls.get_backend_sections()
        study_sections = cls.get_study_sections()

        # filter out commented sections (e.g., [*backend_section:tag])
        config_sections = [section for section in config.keys() if section.split(":")[0][0] != "*"]

        for section in config_sections:
            if Task.get_backend(section) in backend_sections:
                # backend_section:tag
                continue
            if section.startswith((study_sections)):
                continue
            if section not in check_params.keys():
                raise RuntimeError("Wrong section:", section)
            # Check the params for the section
            for param in config[section].keys():
                if param not in check_params[section]:
                    raise RuntimeError("Wrong section param:", section, param)
            for param in check_params[section]:
                if param not in config[section].keys():
                    if not check_params[section][param]['optional']:
                        raise RuntimeError("Missing section param:", section, param)
                    else:
                        # Add the default value for this param
                        config[section][param] = check_params[section][param]['default']
                else:
                    ptype = type(config[section][param])
                    ptype_ok = check_params[section][param]["type"]
                    ptype_default = check_params[section][param]["default"]
                    if ptype != ptype_ok and ptype_default is not None:
                        msg = "Wrong type for section param: %s %s %s should be %s" % \
                              (section, param, ptype, ptype_ok)
                        raise RuntimeError(msg)

        # And now the backend_section entries
        # This only validates the types of each param if present, and doesn't check that
        # all required parameters are set.  This functionality has been moved to the
        # get_backend_section method
        check_params = cls.backend_section_params()
        for section in config_sections:
            if Task.get_backend(section) in backend_sections:
                for param in check_params:
                    if param in config[section].keys():
                        ptype = type(config[section][param])
                        ptype_ok = check_params[param]["type"]
                        if ptype != ptype_ok:
                            msg = "Wrong type for section param: %s %s %s should be %s" % \
                                  (section, param, ptype, ptype_ok)
                            raise RuntimeError(msg)

    def get_backend_section(
        self,
        base_backend_section: str,
        *parameters: str
    ) -> Dict[str, Any]:
        """
        A smart config object that supports subscripting

        Config options are set through an INI file with several sections, such as
        `[gitlab]`, called base backend sections, as well as sections like
        `[gitlab:issue]` or `[gitlab:ieee]`, called parameterized backend sections.

        Backend configuration can be accessed by passing a base backend section, as well
        as a list of parameters.  For example, you can pass `"gitlab", []`,
        `"gitlab", ["issue"]`, or `"gitlab", ["issue", "ieee"]`".  The resulting
        configuration will be the base backend section (`[gitlab]`) composed with
        each of parameterized sections passed.  For example, passing `"gitlab", ["issue",
        "ieee"]`, would yield the `[gitlab]` section + the `[gitlab:issue]` + the
        `[gitlab:ieee]` seciton.

        If either the base backend section or any of the parameterized backend sections
        aren't found, they will be assumed to be empty.

        For compatibility reasons, parameterized backend sections can include multiple
        parameters.  That is, the underlying INI file can have a section
        `[gitlab:issue:ieee]`.  If this section is present, it will be applied after the
        base backend section and all of the parameterized backend sections with only one
        parameter, and only when the passed parameters are EXACTLY `"gitlab", ["issue",
        "ieee"] in that order.

        While calling this method with multiple parameters is encouraged, having
        parameterized backend sections with 2+ parameters in the underlying config is
        discouraged.

        Note that config options that are NOT backends can still be accessed through this
        method, and will be treated as un-parameterized base backend sections (because
        that's what they are)

        This functionality can also be accessed by subscripting a :class:`Config` object.
        For more information, see :func:`__get_item__`
        """
        base_section: Dict[str, Any] = self.conf.get(base_backend_section, dict())

        # Start the output with just the base backend section
        output = {key: value for key, value in base_section.items()}  # Shallow copy

        # Compose all of the parameterized backend sections
        for param in parameters:
            output.update(
                self.conf.get(f'{base_backend_section}:{param}', dict())
            )

        # Compose the multi-parametered backend section (for backwards compatibility)
        if len(parameters) > 1:
            output.update(
                self.conf.get(':'.join([base_backend_section] + list(parameters)), dict())
            )

        # Check that all necessary parameters are present
        if base_backend_section in self.get_backend_sections():
            # If this is a config section for a backend, check for missing parameters
            missing_parameters = [
                param_name
                for param_name, param_props
                in self.backend_section_params().items()
                if not param_props['optional'] and param_name not in output
            ]
        else:
            # Otherwise, this is a general config section, so skip the check
            missing_parameters = []
        if len(missing_parameters):
            # If any parameters were missing, raise a detailed error
            missing_parameters_str = ', '.join(missing_parameters)
            if len(parameters):
                backend_string = ':'.join([base_backend_section] + list(parameters))
                parameterized_backend_sections = ', '.join(f'[{base_backend_section}:{parameter}]' for parameter in parameters)
                raise RuntimeError(
                    f'The backend section {backend_string} is used, but neither the base '
                    f'section ([{base_backend_section}]), nor any of the parameterized backend '
                    f'sections ({parameterized_backend_sections}), nor the full backend '
                    f'string section ([{backend_string}]) have the required parameter(s): '
                    f'{missing_parameters_str}.'
                )
            else:
                raise RuntimeError(
                    f'The backend section {base_backend_section} is used, but '
                    f'required parameter(s) are missing: {missing_parameters_str}.'
                )

        return output

    def __getitem__(self, backend_string: str) -> Dict[str, Any]:
        """
        Access the backend sections using backend strings

        Another way of accessing :func:`get_backend_section`

        A backend string is a string such as `"gitlab:issue:ieee"` that can be used to
        access a backend section.  The string is a series of substrings joined by colons
        (`:`).  The first substring should be the base backend section, and all following
        substrings should be parameters.

        The passed string must not be empty
        """
        assert len(backend_string) > 0, "__get_item__ called on a Config object with a zero-length string"

        args = backend_string.split(':')
        return self.get_backend_section(*args)

    T = TypeVar('T')

    def get(self, backend_string: str, default: T = None) -> Union[Dict[str, Any], T]:
        """
        Wraps :func:`__getitem__` to allow specifying a default value

        The default value is returned any time the output would otherwise be an empty
        dict, for example if neither the base backend section nor any of the parameters are
        present in the config.
        """
        out = self[backend_string]
        if len(out) == 0:
            return default
        else:
            return out

    def __contains__(self, backend_string: str) -> bool:
        """
        Check if any component of the provided backend string is configured

        This returns `True` if
         - The base backend section is present in the underlying config
         - Any of the parameterized versions of the base backend section are present
         - The full multi-parameterized backend section is present

        Read :func:`get_backend_section` for more details about what this means
        """
        split = backend_string.split(':')
        return split[0] in self.conf\
            or any(f'{split[0]}:{param}' in self.conf for param in split[1:])\
            or backend_string in self.conf

    def __add_types(self, raw_conf):
        """ Convert to int, boolean, list, None types config items """

        typed_conf = {}

        for s in raw_conf.keys():
            typed_conf[s] = {}
            for option in raw_conf[s]:
                val = raw_conf[s][option]
                if len(val) > 1 and (val[0] == '"' and val[-1] == '"'):
                    # It is a string
                    typed_conf[s][option] = val[1:-1]
                # Check list
                elif len(val) > 1 and (val[0] == '[' and val[-1] == ']'):
                    # List value
                    typed_conf[s][option] = val[1:-1].replace(' ', '').split(',')
                # Check boolean
                elif val.lower() in ['true', 'false']:
                    typed_conf[s][option] = True if val.lower() == 'true' else False
                # Check None
                elif val.lower() == 'none':
                    typed_conf[s][option] = ''
                else:
                    try:
                        # Check int
                        typed_conf[s][option] = int(val)
                    except ValueError:
                        # Is a string
                        typed_conf[s][option] = val
        return typed_conf

    def _add_to_conf(self, new_conf):
        """Add new configuration to self.conf.

        Adds configuration parameters in new_con to self.conf.
        If they already existed in conf, overwrite them.

        :param new_conf: new configuration, to add
        """

        for section in new_conf:
            if section not in self.conf:
                self.conf[section] = new_conf[section]
            else:
                for param in new_conf[section]:
                    self.conf[section][param] = new_conf[section][param]

    def __read_conf_files(self):
        logger.debug("Reading conf files")
        self.conf = {}
        for conf_file in self.conf_list:
            logger.debug("Reading conf files: %s", conf_file)
            parser = configparser.ConfigParser()
            parser.read(conf_file)
            raw_conf = {s: dict(parser.items(s)) for s in parser.sections()}
            conf = self.__add_types(raw_conf)
            self._add_to_conf(conf)
        self.check_config(self.conf)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2023 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Luis Cañas-Díaz <lcanas@bitergia.com>
#     Alvaro del Castillo <acs@bitergia.com>
#


import argparse
import logging
import logging.handlers
import os
import sys

sys.path.insert(1, '.')

# Quick fix for avoiding weird warnings
# https://github.com/chaoss/grimoirelab-sirmordred/issues/192
import warnings
warnings.filterwarnings("ignore", message="numpy.dtype size changed")
warnings.filterwarnings("ignore", message="numpy.ufunc size changed")

from sirmordred.config import Config
from sirmordred.sirmordred import SirMordred


SLEEPFOR_ERROR = """Error: You may be Arthur, King of the Britons. But you still """ + \
                 """need the 'sleep_for' variable in sortinghat section\n - Sir Mordred said."""

Logger = logging.getLogger(__name__)


def main():
    args = parse_args()

    if args.config_template_file is not None:
        Config.create_config_file(args.config_template_file)
        Logger.info("Sample config file created in {}".format(args.config_template_file))
        return 0
    elif args.config_files is None:
        Logger.error("Option -t or -c is required")
        return 1

    try:
        config = Config(args.config_files[0], args.config_files[1:])
        config_dict = config.get_conf()
        logs_dir = config_dict['general'].get('logs_dir', None)
        debug_mode = config_dict['general']['debug']
        short_name = config_dict['general']['short_name']
        logger = setup_logs(logs_dir, debug_mode, short_name)
    except RuntimeError as error:
        print("Error while consuming configuration: {}".format(error))
        return 1

    if args.phases:
        logger.info("Executing sirmordred for phases: {}".format(args.phases))
        # HACK: the internal dict of Config is modified directly
        # In manual phases execute sirmordred as an script
        config_dict['general']['update'] = False
        for phase in config_dict['phases']:
            config_dict['phases'][phase] = True if phase in args.phases else False

    SirMordred(config).start()


def setup_logs(logs_dir, debug_mode, short_name):

    if debug_mode:
        logging_mode = logging.DEBUG
    else:
        logging_mode = logging.INFO

    logger = logging.getLogger()
    logger.setLevel(logging_mode)

    # create formatter and add it to the handlers
    formatter = logging.Formatter(f'%(asctime)s - {short_name} - %(name)s - %(levelname)s - %(message)s')

    # create file handler
    if logs_dir:
        fh_filepath = os.path.join(logs_dir, 'all.log')
        fh = logging.FileHandler(fh_filepath)
        fh.setLevel(logging_mode)
        # Set format
        fh.setFormatter(formatter)
        # add the handlers to the logger
        logger.addHandler(fh)

    # create console handler
    ch = logging.StreamHandler()
    ch.setLevel(logging_mode)
    ch.setFormatter(formatter)
    logger.addHandler(ch)

    # this is needed because the perceval log messages are similar to ELK/mordred ones
    if not debug_mode:
        logging.getLogger('perceval').setLevel(logging.WARNING)

    logging.getLogger('requests').setLevel(logging.WARNING)
    logging.getLogger('urrlib3').setLevel(logging.WARNING)
    logging.getLogger('elasticsearch').setLevel(logging.WARNING)

    return logger


def parse_args():
    parser = argparse.ArgumentParser(
        description='SirMordred, the friendly friend of GrimoireELK',
        epilog='Software metrics for your peace of mind'
    )

    parser.add_argument('-c', '--config', help='Configuration files',
                        type=str, nargs='+', default=['mordred.cfg'],
                        dest='config_files')
    parser.add_argument('-t', '--template', help='Create template configuration file',
                        dest='config_template_file')
    parser.add_argument('-p', '--phases', nargs='*',
                        help='List of phases to execute (update is set to false)')

    args = parser.parse_args()
    return args


if __name__ == '__main__':
    try:
        code = main()
        sys.exit(code)
    except KeyboardInterrupt:
        s = "\n\nReceived Ctrl-C or other break signal. Exiting.\n"
        sys.stderr.write(s)
        sys.exit(0)
    except RuntimeError as e:
        s = "Error: %s\n" % str(e)
        sys.stderr.write(s)
        sys.exit(1)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#   Quan Zhou <quan@bitergia.com>
#


import argparse
import json
import yaml


def write_yaml(file_name, data):
    with open(file_name, 'w+') as f:
        yaml.dump(data, f, default_flow_style=False)


def open_file(file_name):
    data = open(file_name).read()
    json_data = json.loads(data)

    return json_data


def read_arguments():
    desc = "Convert JSON to YML: return hierarchy.yml and projects_repo.yml"
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=desc)

    parser.add_argument("json_file",
                        action="store",
                        help="JSON file: input")

    args = parser.parse_args()

    return args


def get_hierarchy_list(json_data):
    hierarchy_list = {}
    for data in json_data["projects"]:
        if len(json_data["projects"][data]['parent_project']) == 0:
            hierarchy_list[data] = []
        else:
            for key in json_data["projects"][data]['parent_project']:
                hierarchy_list[data][key] = []

    return hierarchy_list


def get_repo_list(json_data, not_backend, special_backend):
    repo_to_return = {}
    for data in json_data["projects"]:
        repo_to_return[data] = {"meta": {"title": json_data["projects"][data]["title"].lower()}}
        for backend_name in json_data["projects"][data]:
            backend = json_data["projects"][data][backend_name]

            if len(backend) > 0 and backend_name not in not_backend or backend_name == "gerrit_repo" and len(backend[0]) > 0:
                repo_list = []
                for repo in backend:
                    if backend_name not in special_backend:
                        repo_list.append(repo['url'])
                    else:
                        repo_list.append(repo['url'] + " " + repo['path'])
                repo_to_return[data][backend_name] = repo_list

    return repo_to_return


if __name__ == "__main__":
    args = read_arguments()
    json_file = args.json_file
    json_data = open_file(json_file)

    not_backend = ["title", "description", "dev_list", "gerrit_repo"]
    special_backend = ["irc", "supybot", "mbox"]

    hierarchy_list = get_hierarchy_list(json_data)
    repo_list = get_repo_list(json_data, not_backend, special_backend)

    write_yaml("hierarchy.yml", hierarchy_list)
    write_yaml("projects-repos.yml", repo_list)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2019 Bitergia
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#
# Authors:
#     Alvaro del Castillo <acs@bitergia.com>


import logging
import requests
import sys
import unittest

from os.path import expanduser, join

# Hack to make sure that tests import the right packages
# due to setuptools behaviour
sys.path.insert(0, '..')

from sirmordred.config import Config
from sirmordred.task_collection import TaskRawDataCollection
from sirmordred.task_projects import TaskProjects

CONF_FILE = 'test.cfg'
PROJ_FILE = 'test-projects.json'

CONF_FILE_NO_COLL = 'test-no-collection.cfg'
PROJ_FILE_NO_COLL = 'test-projects-no-collection.json'

CONF_ARCHIVE_FILE = 'archives-test.cfg'

HOME_USER = expanduser("~")
PERCEVAL_ARCHIVE = join(HOME_USER, '.perceval')

GIT_BACKEND_SECTION = 'git'

GITHUB_BACKEND_SECTION = 'github'
GITHUB_REPO = "https://github.com/grimoirelab/perceval"


logging.basicConfig(level=logging.INFO)


class TestTaskRawDataCollection(unittest.TestCase):
    """Task tests"""

    def test_initialization(self):
        """Test whether attributes are initializated"""

        config = Config(CONF_FILE)
        backend_section = GIT_BACKEND_SECTION
        task = TaskRawDataCollection(config, backend_section=backend_section)

        self.assertEqual(task.config, config)
        self.assertEqual(task.backend_section, backend_section)

    def test_backend_params(self):
        """Test whether the backend parameters are initializated"""

        config = Config(CONF_FILE)
        backend_section = GITHUB_BACKEND_SECTION
        task = TaskRawDataCollection(config, backend_section=backend_section)
        params = task._compose_perceval_params(GITHUB_BACKEND_SECTION, GITHUB_REPO)

        expected_params = [
            'grimoirelab',
            'perceval',
            '--api-token',
            'XXXXX',
            '--sleep-time',
            '300',
            '--sleep-for-rate',
            '--category',
            'issue',
            '--archive-path',
            '/tmp/test_github_archive'
        ]

        self.assertEqual(len(params), len(expected_params))

        for p in params:
            self.assertTrue(p in expected_params)

    def test_execute(self):
        """Test whether the Task could be run"""

        config = Config(CONF_FILE)
        cfg = config.get_conf()
        backend_section = GIT_BACKEND_SECTION
        task = TaskRawDataCollection(config, backend_section=backend_section)
        # We need to load the projects
        TaskProjects(config).execute()
        self.assertIsNotNone(task.execute())

        # Check that the collection went well
        es_collection = cfg['es_collection']['url']
        raw_index = es_collection + "/" + cfg[GIT_BACKEND_SECTION]['raw_index']

        r = requests.get(raw_index + "/_search?size=0", verify=False)
        total = r.json()['hits']['total']
        raw_items = total['value'] if isinstance(total, dict) else total
        self.assertEqual(raw_items, 3603)

    def test_execute_no_collection(self):
        """Test whether the raw data is not downloaded when --filter-no-collection is true"""

        config = Config(CONF_FILE_NO_COLL)
        cfg = config.get_conf()
        backend_section = GIT_BACKEND_SECTION
        task = TaskRawDataCollection(config, backend_section=backend_section)
        # We need to load the projects
        TaskProjects(config).execute()
        self.assertIsNotNone(task.execute())

        # Check that the fitler --filter-no-collection works
        es_collection = cfg['es_collection']['url']
        raw_index = es_collection + "/" + cfg[GIT_BACKEND_SECTION]['raw_index']

        r = requests.get(raw_index + "/_search?size=0", verify=False)
        total = r.json()['hits']['total']
        raw_items = total['value'] if isinstance(total, dict) else total
        self.assertEqual(raw_items, 40)

    def test_execute_filter_no_collection(self):
        """Test whether the Task could be run"""

        config = Config(CONF_FILE)
        backend_section = GIT_BACKEND_SECTION
        task = TaskRawDataCollection(config, backend_section=backend_section)
        # We need to load the projects
        TaskProjects(config).execute()
        self.assertIsNotNone(task.execute())

    def test_execute_from_archive(self):
        """Test fetching data from archives"""

        # proj_file -> 'test-projects-archive.json' stored within the conf file
        config = Config(CONF_ARCHIVE_FILE)

        backend_sections = ['askbot', 'bugzilla', 'bugzillarest', 'confluence',
                            'discourse', 'dockerhub', 'gerrit', 'github:issue', 'github:pull',
                            'gitlab:issue', 'gitlab:merge', 'google_hits', 'jenkins',
                            'jira', 'mediawiki', 'meetup', 'mozillaclub', 'nntp', 'phabricator',
                            'redmine', 'remo', 'rss', 'stackexchange', 'slack', 'telegram', 'twitter']

        # We need to load the projects
        TaskProjects(config).execute()
        for backend_section in backend_sections:
            task = TaskRawDataCollection(config, backend_section=backend_section)
            errors = task.execute()
            for err in errors:
                self.assertIn('backend', err)
                self.assertIn('repo', err)
                self.assertIn('error', err)
                self.assertIsNone(err['error'])


if __name__ == "__main__":
    unittest.main(warnings='ignore')
